@@ -17,10 +17,11 @@
 #      along with this program.
 #
 #  File: command.tcl
-#  Last change:  9 March 2004
+#  Last change:  7 October 2004
 #
 
-# this file is only source-ed in command-line mode
+# this file is only source-ed in command-line mode (even if the graphical
+#  interface is launched to obey the command)
 
 ### change some initial values if needs be
 
@@ -38,7 +39,8 @@ if { $COMMAND(log) } {
 proc ExecCommand {} {
     # execute command from command line
     # return 0 on success, 1 on error, "wait" to loop, "-0" to quit fast
-    global COMMAND GPSProtocol PVTState RECPROTS GPSREC NoImgLib GSHPVersion
+    global COMMAND CMDLINE NoImgLib GSHPVersion RealTimeLogAnim PVTState \
+	 PVTDumping RealTimeGettingFix MESS
 
     switch $COMMAND(name) {
 	checkconn {
@@ -83,18 +85,40 @@ proc ExecCommand {} {
 	    }
 	}
 	getrtimelog {
-	    if { [set prot $COMMAND(prot)] == "" && \
-		    [catch {set prot $RECPROTS($GPSREC)}] } {
-		return 1
-	    }
-	    # this depends on the Garmin variant...
-	    if { $prot == "garmin" && [ExecGPSConnectFail] } { return 1 }
-	    set GPSProtocol $prot	    
-	    if { ! [StartRealTimeLog $COMMAND(interv)] || \
-		    [CmdRTimeDumpFails $COMMAND(path)] } { return 1 }
+	    if { [CmdCantStartRTimeLog Ignore Ignore] || \
+		     [CmdRTimeDumpFails $COMMAND(path)] } { return 1 }
+	    # force calls to SavePVTData $PVTDFile
+	    set PVTDumping 1
 	    set PVTState on
 	    return wait
 	}
+	getfix {
+	    # interval for real-time log
+	    set COMMAND(interv) 2
+	    if { [CmdCantStartRTimeLog Ignore Ignore] || \
+		     [CmdRTimeDumpFails $COMMAND(path)] } { return 1 }
+	    set RealTimeGettingFix 1
+	    trace variable RealTimeGettingFix w CmdRTimeGotFix
+	    if { $COMMAND(timeout) != 0 } {
+		after [expr $COMMAND(timeout)*1000] \
+		    { set RealTimeGettingFix -1 }
+	    }
+	    # avoid calls to SavePVTData $PVTDFile
+	    set PVTDumping 0
+	    set PVTState on
+	    return wait
+	}
+	starttravel {
+	    # enter graphical mode
+	    set CMDLINE 0
+	    GMInit
+	    if { ! [CmdCantStartRTimeLog EnableGPS \
+			[list GMMessage $MESS(receiver)]] } {
+		set RealTimeLogAnim 1
+		ToTravel
+	    }
+	    return wait
+	}
 	project {
 	    if { [CmdBadProjArgs] || [CmdBadProjSetup] } { return 1 }
 	    set proj $COMMAND(proj)
@@ -192,15 +216,36 @@ proc CmdInterpret {file} {
     return 0
 }
 
-proc ExecGPSConnectFail {} {
+proc ExecGPSConnectFail {args} {
+    #  $args is list with 1st and 2nd argument to proc GPSConnection
     # return 1 on error, 0 on success
     global GPSOpResult
 
-    if { [GPSConnection Ignore Ignore] == 0 } { return 1 }
+    if { $args == 0 } {
+       if { [GPSConnection] == 0 } { return 1 }
+    } elseif { [GPSConnection [lindex $args 0] [lindex $args 1]] == 0 } {
+	return 1
+    }
     vwait GPSOpResult
     return $GPSOpResult
 }
 
+proc CmdCantStartRTimeLog {connproc errorproc} {
+    # return 0 on success, 1 on error
+    global COMMAND RECPROTS GPSREC GPSProtocol
+
+    if { [set prot $COMMAND(prot)] == "" } {
+	set prot $GPSProtocol
+    } else {
+	set GPSProtocol $prot
+    }
+    # this depends on the Garmin variant...
+    if { $prot == "garmin" && \
+	     [ExecGPSConnectFail $connproc $errorproc] } { return 1 }
+    if { ! [StartRealTimeLog $COMMAND(interv)] } { return 1 }
+    return 0
+}
+
 proc CmdGetFails {types} {
     # get items of given $types
     # return 0 on success
@@ -333,8 +378,8 @@ proc CmdWriteFails {fmt path pars types} {
 			return 1
 		    }
 		}
+		if { $path != "stdout" } { close $f }
 	    }
-	    if { $path != "stdout" } { close $f }
 	}
     }
     return 0
@@ -372,14 +417,47 @@ proc CmdProcParamsFails {fmt pars} {
 proc CmdRTimeDumpFails {path} {
     # start dumping real-time log continuously
     # return 0 on success
-    global PVTDFile PVTDumping
+    global PVTDFile
 
     if { [set PVTDFile [CmdOpenFile $path out]] == -1 } { return 1 }
-    set PVTDumping 1
-    # SavePVTData $PVTDFile
     return 0
 }
 
+proc CmdRTimeGotFix {args} {
+    # global  RealTimeGettingFix has been changed to either -1 for a timeout
+    #  or a list with formatted real-time position data --- see procs
+    #  UseRealTimeData and RTimeDumpLine
+    # exit with 1 on timeout
+    # if the fix is valid write to $PVTDFile and exit with 0, otherwise
+    #  reset RealTimeGettingFix to 1
+    global RealTimeGettingFix PVTDFile TXT DSCALE SPUNIT
+
+    if { $RealTimeGettingFix == -1 } { exit 1 }
+    foreach "n date lat long alt fix epe eph epv velx vely velz trk" \
+	$RealTimeGettingFix { break }
+    if { $fix != $TXT(posfixerror) && $fix != $TXT(posfix_) } {
+	puts $PVTDFile "$lat $long"
+	puts $PVTDFile "WGS 84"
+	puts $PVTDFile $date
+	puts $PVTDFile $trk
+	# horizontal speed
+	if { $velx != "" && $vely != "" } {
+	    puts $PVTDFile [format "%.2f $SPUNIT" \
+				[expr sqrt($velx*$velx+$vely*$vely)*$DSCALE]]
+	} else { puts $PVTDFile "" }
+	if { $velz != "" } {
+	    puts $PVTDFile [format "%.1f" $velz]
+	} else { puts $PVTDFile "" }
+	puts $PVTDFile $epe
+	puts $PVTDFile $eph
+	puts $PVTDFile $epv
+	close $PVTDFile
+	exit 0
+    }
+    set RealTimeGettingFix 1
+    return
+}
+
 proc CmdShow {what} {
     # print information to stdout
     global VERSION FILEFORMAT RECPROTS TXT MESS MAPKNOWNPROJS MAPPROJDATA \
