@@ -17,18 +17,20 @@
 #      along with this program.
 #
 #  File: garmin.tcl
-#  Last change:  7 April 2004
+#  Last change:  22 October 2004
+#
+# Includes contributions by
+#  - David Gardner (djgardner@users.sourceforge.net) marked "DJG contribution"
 #
 
  # implemented using
  #  "Garmin GPS Interface Specification", May 6 1998, 001-00063-00 Rev. 1
  #  kindly provided by Garmin Corporation
- # updated to follow the description in
+ # updated to follow the descriptions available from www.garmin.com:
  #  "Garmin GPS Interface Specification", April 13 1999, 001-00063-00 Rev. 2
- #  available from www.garmin.com
- # updated to follow the description in
  #  "Garmin GPS Interface Specification", December 6 1999, 001-00063-00 Rev. 3
- #  available from www.garmin.com
+ #  "Garmin GPS Interface Specification", April 24 2004, 001-00063-00 Rev. A
+
 
 
 # logging the communication: see serial.tcl
@@ -128,7 +130,7 @@ proc DoSendPacket {} {
     global SInPacketState SInState SOutBusy RPID DLE DLECHR ETX ACK NAK \
 	    SOutPID SOutBData SOutDSum SOutBPID SOutSize SRLFILE
 
-    if { $SOutBPID!=$ACK && $SOutBPID!=$NAK && "$SInState" == "get" } {
+    if { $SOutBPID!=$ACK && $SOutBPID!=$NAK && $SInState == "get" } {
 	after 50 DoSendPacket
         return
     }
@@ -149,7 +151,7 @@ proc DoSendPacket {} {
     }
     foreach b $SOutBData {
 	puts -nonewline $SRLFILE "$b"
-	if { "$b" == "$DLECHR" } {
+	if { $b == $DLECHR } {
 	    puts -nonewline $SRLFILE "$DLECHR"
 	}
     }
@@ -277,7 +279,7 @@ proc ProcChar {char} {
     }
     set dec [expr ($dec+256)%256]
     Log "PC> got $dec; state=$SInState, $SInPacketState"
-    if { "$SInState" == "idle" && ! $GetPVT } {
+    if { $SInState == "idle" && ! $GetPVT } {
 	    Log "PC> idle: discarded $dec" ; return
     }
     switch $SInPacketState {
@@ -360,7 +362,7 @@ proc PacketOK {} {
     global SInBuffer SInSum SInCount PacketID PacketData
 
     set PacketData ""
-    if { "$PacketID" == "error" } { return 0 }
+    if { $PacketID == "error" } { return 0 }
     set size [expr $SInCount-2]
     binary scan [lindex $SInBuffer 0] "c" expsize
     set expsize [expr ($expsize+256)%256]
@@ -394,7 +396,7 @@ proc ProcInPacket {pid data} {
 	    PDTYPE SPckts GetPVT PkLastPID PkLastData Jobs \
 	    TestRepeated PrdDATA
 
-    if { "$pid" == "PVTData" } {
+    if { $pid == "PVTData" } {
 	# PVT data protocol
 	if { ! $GetPVT } {
 	    Log "PP> discarding PVT packet"
@@ -405,7 +407,7 @@ proc ProcInPacket {pid data} {
 	return
     }
     if { $TestRepeated && \
-	    "$pid" == "$PkLastPID" && "$data" == "$PkLastData" } {
+	    $pid == $PkLastPID && $data == $PkLastData } {
 	Log "PP> discarding repeated packet for $pid"
 	return
     }
@@ -443,14 +445,14 @@ proc ProcInPacket {pid data} {
 	    ExpectRecords $pid $data WPData
 	}
 	A100EXF {
-	    # expecting end of tranfer on A100 protocol
-	    EndOfTranfer $pid $data WP A100 $PkInData
+	    # expecting end of transfer on A100 protocol
+	    EndOfTransfer $pid $data WP A100 $PkInData
 	}
 	D100 -  D101 -  D102 -  D103 -  D104 -  D105 -  D106 -
-	D107 -  D108 -  D109 -  D150 -  D151 -  D152 -  D154 -  D155 {
+	D107 -  D108 -  D109 -  D110 -  D150 -  D151 -  D152 -  D154 -  D155 {
 	    # WP data; product specific
-	    if { "$PkInPrevState" == "$CurrPSPID(RT)" } {
-		if { "$pid" == "RTHeader" } {
+	    if { $PkInPrevState == $CurrPSPID(RT) } {
+		if { $pid == "RTHeader" } {
 		    lappend PkInRTsData $PkInData \
 			"[UnPackData $data $PDTYPE($CurrPSPID(RTHeader))]"
 		    set PkInData ""
@@ -482,14 +484,14 @@ proc ProcInPacket {pid data} {
 	    set PkInRTsData ""
 	}
 	A200EXF {
-	    # expecting end of tranfer on A200 protocol
+	    # expecting end of transfer on A200 protocol
 	    lappend PkInRTsData $PkInData
-	    EndOfTranfer $pid $data RT A200 $PkInRTsData
+	    EndOfTransfer $pid $data RT A200 $PkInRTsData
 	}
 	A201EXF {
-	    # expecting end of tranfer on A201 protocol
+	    # expecting end of transfer on A201 protocol
 	    lappend PkInRTsData $PkInData
-	    EndOfTranfer $pid $data RT A201 $PkInRTsData
+	    EndOfTransfer $pid $data RT A201 $PkInRTsData
 	}
 	D200 -  D201 -  D202 {
 	    # RT header; product specific
@@ -507,7 +509,7 @@ proc ProcInPacket {pid data} {
 	}
 	D210 {
 	    # RT link type; product specific
-	    if { "$pid" == "RTHeader" } {
+	    if { $pid == "RTHeader" } {
 		lappend PkInRTsData $PkInData \
 			"[UnPackData $data $PDTYPE($CurrPSPID(RTHeader))]"
 		set PkInData ""
@@ -534,21 +536,22 @@ proc ProcInPacket {pid data} {
 	    # TR transfer protocol
 	    ExpectRecords $pid $data TRData
 	}
-	A301 {
+	A301 -  A302 {
 	    # TR transfer protocol
 	    ExpectRecords $pid $data TRHeader
 	}
 	A300EXF {
-	    # expecting end of tranfer on A300 protocol
-	    EndOfTranfer $pid $data TR A300 $PkInData
+	    # expecting end of transfer on A300 protocol
+	    EndOfTransfer $pid $data TR A300 $PkInData
 	}
-	A301EXF {
-	    # expecting end of tranfer on A301 protocol
-	    EndOfTranfer $pid $data TR A301 $PkInData
+	A301EXF -  A302EXF {
+	    # expecting end of transfer on A301, A302 protocols
+	    regsub {EXF$} $PkInState "" prot
+	    EndOfTransfer $pid $data TR $prot $PkInData
 	}
-	D300 -  D301 {
+	D300 -  D301 -  D302 {
 	    # TR data; product specific
-	    if { "$pid" == "TRHeader" } {
+	    if { $pid == "TRHeader" } {
 		set PkInState $CurrPSPID(TRHeader)
 		ProcInPacket $pid $data
 		return
@@ -560,7 +563,7 @@ proc ProcInPacket {pid data} {
 		} else { incr PkInCount -1 }
 	    }
 	}
-	D310 {
+	D310 -  D311 -  D312 {
 	    # TR header; product specific
 	    if { [GoodPID $pid TRHeader] } {
 		lappend PkInData \
@@ -578,8 +581,8 @@ proc ProcInPacket {pid data} {
 	    ExpectRecords $pid $data ALData
 	}
 	A500EXF {
-	    # expecting end of tranfer on A500 protocol
-	    EndOfTranfer $pid $data AL A500 $PkInData
+	    # expecting end of transfer on A500 protocol
+	    EndOfTransfer $pid $data AL A500 $PkInData
 	}
 	D500 -  D501 {
 	    # AL data; product specific
@@ -604,8 +607,25 @@ proc ProcInPacket {pid data} {
 		EndInProt D700 [UnPackData $data $PDTYPE($CurrPSPID(PosnData))]
 	    }
 	}
+	A906 {
+	    # LAP transfer protocol (from receiver)
+	    ExpectRecords $pid $data LAPData
+	}
+	A906EXF {
+	    # expecting end of transfer on A906 protocol
+	    EndOfTransfer $pid $data LAP A906 $PkInData
+	}
+	D906 {
+	    # LAP data; product specific
+	    if { [GoodPID $pid LAPData] } {
+		lappend PkInData [UnPackData $data $PDTYPE($PkInState)]
+		if { $PkInCount == 1 } {
+		    set PkInState ${PkInPrevState}EXF
+		} else { incr PkInCount -1 }
+	    }
+	}
 	A011 -
-	A400 -
+	A400 -  A600 -  A650 -  A700 -
 	default {
 	    Log "$PkInState not supported"
 	    set PkInState idle
@@ -631,7 +651,7 @@ proc ExpectRecords {pid data type} {
     return
 }
 
-proc EndOfTranfer {pid data wh prot recs} {
+proc EndOfTransfer {pid data wh prot recs} {
     # end of transfer after records for protocol $prot
     #  $recs is data collected from the records
     global PkInData
@@ -648,7 +668,7 @@ proc EndOfTranfer {pid data wh prot recs} {
 proc GoodPID {pid wanted} {
     # check that packet id name is what is wanted
 
-    if { "$pid" != "$wanted" } {
+    if { $pid != $wanted } {
 	BadPacket "PID $pid instead of $wanted"
 	return 0
     }
@@ -682,7 +702,10 @@ proc SendData {type args} {
     global PDTYPE PRTHNID PkInState SOutBusy CurrPSPID CMD Command CommArgs \
 	    PkOutState PkOutData PkOutCount PkOutStart PkOutWhat PkTRSegs \
 	    PkOutSaved RTIdNumber RTWPoints RTStages TRTPoints TRSegStarts \
-	    PrdDATA MESS
+	    PrdDATA MESS PTRHNID
+    # DJG contribution
+    global RTNextNumber AutoNumRts RTSendNumber
+    #--
 
     set Command $type ; set CommArgs $args
     switch $type {
@@ -711,7 +734,7 @@ proc SendData {type args} {
 	}
 	put {
 	    set putwhat [lindex $args 0]
-	    if { "$putwhat" == "Prx" || "$putwhat" == "AL" } {
+	    if { $putwhat == "Prx" || $putwhat == "AL" } {
 		Log "put $putwhat not implemented"
 		return
 	    }
@@ -738,12 +761,48 @@ proc SendData {type args} {
 		    set ixs [lindex $args 1]
 		    set n 0 ; set badrts 0
 		    set data ""
-		    foreach rt $ixs {
-			if { $PRTHNID && \
-				! [regexp {^[0-9]+$} $RTIdNumber($rt)] } {
-			    incr badrts
-			    continue
+		    # DJG contribution: do autonumbering of RTs if the device 
+		    # can't cope with names; and try to do it the proper way...
+		    # If the user has numbered their routes (silly thing to
+		    # do in my book, as my GPS can only cope with so many
+		    # of the things...) then we obey that. Otherwise we
+		    # assign numbers in a first-come first served way. If
+		    # some routes are numbered, then we cope with that too by
+		    # preallocating those numbers 
+		    if ($AutoNumRts) {
+			foreach rt [ array names RTIdNumber ] {
+			    set num $RTIdNumber($rt) 
+			    if { [regexp {^[0-9]+$} $RTIdNumber($rt)] } {
+				set $taken($num) 1;
+				set $RTSendNumber($rt) $num
+			    }
 			}
+		    }
+		    #--
+		    foreach rt $ixs {
+			# DJG contribution: auto-numbering
+			if { $PRTHNID } {
+			    if { ! [regexp {^[0-9]+$} $RTIdNumber($rt)] } {
+				# AutoNumRts is a user option
+			    	if { ! $AutoNumRts } {
+				    incr badrts
+				    continue
+				} else {
+				    if {  [ catch {set RTSendNumber($rt)} ] } {
+					while { ! [catch \
+						{set taken($RTNextNumber)}] } {
+					    incr RTNextNumber
+					}
+					set RTSendNumber($rt) $RTNextNumber
+					incr RTNextNumber
+				    }
+				}
+			    }
+			    # Alternative taken care of above.
+			} else {
+			    set RTSendNumber($rt) $RTIdNumber($rt)
+			}
+			#--
 			set k [llength $RTWPoints($rt)]
 			if { ! $nolk } {
 			    incr k [expr $k-1]
@@ -771,14 +830,22 @@ proc SendData {type args} {
 		}
 		TR {
 		    set ixs [lindex $args 1]
-		    set n 0
+		    set n 0 ; set badtrs 0
 		    set PkTRSegs 0
 		    foreach tr $ixs {
+			if { $PTRHNID && ! [regexp {^[0-9]+$} $TRName($ix)] } {
+			    incr badtrs
+			    continue
+			}
 			incr n [llength $TRTPoints($tr)]
-			if { "$TRSegStarts($tr)" != "" } {
+			if { $TRSegStarts($tr) != "" } {
 			    incr PkTRSegs
 			}
 		    }
+		    if { $badtrs > 0 } {
+			GMMessage [format $MESS(cantsaveTRid) $badtrs]
+			if { $n == 0 } return
+		    }
 		    set PkOutStart XfrTR
 		    set PkOutData $ixs
 		    switch $p {
@@ -788,7 +855,7 @@ proc SendData {type args} {
 			    set PkOutCount $n
 			    set PkOutState TRData
 			}
-			A301 {
+			A301 -  A302 {
 			    # $PkTRSegs is count of TRs having segments
 			    set PkOutCount [expr $n+[llength $ixs]]
 			    set PkOutState TRHeader
@@ -822,7 +889,7 @@ proc SentPacket {pid} {
     Log "StP> packet for $pid sent"
     switch $pid {
 	PrdReq {
-	    if { "$PkInState" != "idle" } {
+	    if { $PkInState != "idle" } {
 		lappend Jobs [after 50 "SentPacket $pid"]
 		return
 	    }
@@ -832,7 +899,7 @@ proc SentPacket {pid} {
 	CmdData {
 	    switch $Command {
 		get {
-		    if { "$PkInState" != "idle" } {
+		    if { $PkInState != "idle" } {
 			lappend Jobs [after 50 "SentPacket $pid"]
 			return
 		    }
@@ -891,7 +958,7 @@ proc SentPacket {pid} {
 	    set PkOutData [Apply "$RTWPoints($rt)" IndexNamed WP]
 	    if { ! $nolk } {
 		set k 1
-		if { "$RTStages($rt)" == "" } {
+		if { $RTStages($rt) == "" } {
 		    foreach nwp [lrange $PkOutData 1 end] {
 			set PkOutData [linsert $PkOutData $k ""]
 			incr k 2
@@ -923,9 +990,9 @@ proc SentPacket {pid} {
 		}
 	    } else {
 		incr PkOutCount -1
-		if { "$CurrPSPID(RT)" != "A201" } {
+		if { $CurrPSPID(RT) != "A201" } {
 		    SendPacket RTWPData [PrepData]
-		} elseif { "$pid" != "RTLinkData" } {
+		} elseif { $pid != "RTLinkData" } {
 		    set PkOutState $CurrPSPID(RTLinkData)
 		    SendPacket RTLinkData [PrepData]
 		} else {
@@ -949,9 +1016,9 @@ proc SentPacket {pid} {
 		SendPacket XfrCmpl [list int $CMD($PkOutStart)]
 	    } else {
 		incr PkOutCount -1
-		if { "$PkOutSaved" == "" } {
+		if { $PkOutSaved == "" } {
 		    switch $CurrPSPID(TR) {
-			A301 {
+			A301 -  A302 {
 			    set pid TRHeader
 			    set PkOutState $CurrPSPID(TRHeader)
 			}
@@ -973,16 +1040,19 @@ proc PrepData {} {
 	    WPName WPCommt WPPosn WPDatum WPDate WPSymbol WPDispOpt WPAlt \
 	    WPHidden RTIdNumber RTCommt TRName TRDatum TRTPoints TRSegStarts \
 	    TRColour TRHidden DataIndex
+    #DJG contribution: Autonumber globals
+    global RTSendNumber RTsSent
+    #--
 
     switch $PkOutState {
 	D100 -	D101 -	D102 -	D103 -	D104 -	D105 -	D106 -	D107 -
-	D108 -	D109 -  D150 -  D151 -	D152 -	D154 -  D155 {
+	D108 -	D109 -  D110 -  D150 -  D151 -	D152 -	D154 -  D155 {
 	    set ix [lindex $PkOutData 0]
 	    set PkOutData [lreplace $PkOutData 0 0]
 	    set p $WPPosn($ix)
-	    if { "$WPDatum($ix)" != "WGS 84" } {
-		set p [ConvertDatum [lindex $p 0] [lindex $p 1] \
-			            $WPDatum($ix) "WGS 84" DDD]
+	    if { $WPDatum($ix) != "WGS 84" } {
+		set p [ToDatum [lindex $p 0] [lindex $p 1] \
+			   $WPDatum($ix) "WGS 84"]
 	    }
 	    return [list $PDTYPE($PkOutState) \
 		             [PrepWPData $PkOutState \
@@ -993,9 +1063,10 @@ proc PrepData {} {
 	D200 -  D201 -  D202 {
 	    # RT header
 	    set ix [lindex $PkOutData 0]
+	    # DJG contribution: Use RTSendNumber in place of RTIdNumber
 	    return [list $PDTYPE($PkOutState) \
 		       [PrepRTHeaderData $PkOutState \
-		            $RTIdNumber($ix) $RTCommt($ix)]]
+		            $RTSendNumber($ix) $RTCommt($ix)]]
 	}
 	D210 {
 	    # RT link
@@ -1003,18 +1074,18 @@ proc PrepData {} {
 	    set PkOutData [lreplace $PkOutData 0 0]
 	    return [list $PDTYPE($PkOutState) [PrepRSData $PkOutState $st]]
 	}
-	D300 -  D301 {
-	    if { "$PkOutSaved" != "" } {
+	D300 -  D301 -  D302 {
+	    if { $PkOutSaved != "" } {
 		# $PkOutSaved is list of TPs
 		set p [lindex $PkOutSaved 0]
 		set alt [lindex $p $DataIndex(TPalt)]
 		set depth [lindex $p $DataIndex(TPdepth)]
 		set PkOutSaved [lreplace $PkOutSaved 0 0]
 		set new 0
-		if { $PkTRSegs && "$PkSegData" != "" && \
+		if { $PkTRSegs && $PkSegData != "" && \
 			[lindex $PkSegData 0] == $PkTRTPn } {
 		    set new 1
-		    if { "[set PkSegData [lreplace $PkSegData 0 0]]" == "" } {
+		    if { [set PkSegData [lreplace $PkSegData 0 0]] == "" } {
 			incr PkTRSegs -1
 		    }
 		}
@@ -1031,9 +1102,9 @@ proc PrepData {} {
 		}
 		set alt [lindex $p $DataIndex(TPalt)]
 		set depth [lindex $p $DataIndex(TPdepth)]
-		if { "$TRDatum($ix)" != "WGS 84" } {
-		    set p [ConvertDatum [lindex $p 0] [lindex $p 1] \
-			                $TRDatum($ix) "WGS 84" DDD]
+		if { $TRDatum($ix) != "WGS 84" } {
+		    set p [ToDatum [lindex $p 0] [lindex $p 1] \
+			       $TRDatum($ix) "WGS 84"]
 		    set PkOutSaved [ChangeTPsDatum $PkOutSaved \
 			                           $TRDatum($ix) "WGS 84"]
 		}
@@ -1043,7 +1114,7 @@ proc PrepData {} {
 	    return [list $PDTYPE($PkOutState) \
 		             [PrepTRData $PkOutState $p 0 $new $alt $depth]]
 	}
-	D310 {
+	D310 -  D311 -  D312 {
 	    # TRHeader
 	    set ix [lindex $PkOutData 0]
 	    set PkOutData [lreplace $PkOutData 0 0]
@@ -1052,7 +1123,7 @@ proc PrepData {} {
 	    if { $PkTRSegs } {
 		set PkSegData $TRSegStarts($ix)
 	    }
-	    if { "$TRDatum($ix)" != "WGS 84" } {
+	    if { $TRDatum($ix) != "WGS 84" } {
 		set PkOutSaved [ChangeTPsDatum $PkOutSaved \
 			                      $TRDatum($ix) "WGS 84"]
 	    }
@@ -1064,18 +1135,20 @@ proc PrepData {} {
 }
 
 proc PrepWPData {pid name commt posn date symbol dispopt alt hidden} {
-    #  $symbol only used in D101-9, D154-5
+    #  $symbol only used in D101-9, D110, D154-5
     #  $dispopt only in D103-4, D107-9, D155
-    #  $alt only in D108-9 (as float), D150-5 (as integer)
+    #  $alt only in D108-9, D110 (as float), D150-5 (as integer)
     global SYMBOLCODE DISPOPTCODE
 
     set sc $SYMBOLCODE($symbol)
     set do $DISPOPTCODE($dispopt)
-    if { "$alt" == "" } {
-	if { "$pid" == "D108" || "$pid" == "D109" } {
+    if { $alt == "" } {
+	if { [regexp {^D1((0[89])|10)} $pid] } {
+	    # D108-D110
 	    set alt 1.0e25
 	} else { set alt 0 }
-    } elseif { "$pid" != "D108" || "$pid" == "D109" } {
+    } elseif { ! [regexp {^D1((0[89])|10)} $pid] } {
+	    # not D108-D110
 	set alt [expr round($alt)]
     }
     switch $pid {
@@ -1112,6 +1185,13 @@ proc PrepWPData {pid name commt posn date symbol dispopt alt hidden} {
 	    set dclr [expr ($do<<5)|($colour&0x1f)]
 	    set vs [list $dclr $sc $posn $alt 0 $name $commt]
 	}
+	D110 {
+	    set ps "2 4 6 7 9 16 17"
+	    ### merging display option and colour
+	    set colour [HiddenFindVal D110 colour $hidden]
+	    set dclr [expr ($do<<5)|($colour&0x1f)]
+	    set vs [list $dclr $sc $posn $alt 0 $name $commt]
+	}
 	D150 {
 	    set ps "0 3 4 8" ; set vs [list $name $posn $alt $commt]
 	}
@@ -1164,9 +1244,12 @@ proc PrepRSData {pid stage} {
 proc PrepTRHeader {pid name colour hidden} {
 
     switch $pid {
-	D310 {
+	D310 -  D312 {
 	    set ps "0 2" ; set vs [list [ToGarminColour $pid $colour] $name]
 	}
+	D311 {
+	    set ps 0 ; set vs $name
+	}
     }
     return [MergeData [HiddenVals $pid $hidden] $ps $vs]
 }
@@ -1178,10 +1261,16 @@ proc PrepTRData {pid posn date new alt depth} {
 	    return [list $posn $date $new]
 	}
 	D301 {
-	    if { "$alt" == "" } { set alt 1.0e25 }
-	    if { "$depth" == "" } { set depth 1.0e25 }
+	    if { $alt == "" } { set alt 1.0e25 }
+	    if { $depth == "" } { set depth 1.0e25 }
 	    return [list $posn $date $alt $depth $new]
 	}
+	D302 {
+	    if { $alt == "" } { set alt 1.0e25 }
+	    if { $depth == "" } { set depth 1.0e25 }
+	    # temperature is always sent as undefined
+	    return [list $posn $date $alt $depth 1.0e25 $new]
+	}
     }
 }
 
@@ -1225,7 +1314,7 @@ proc UnPackData {data types} {
 		# this only works with machines following the
 		#  IEEE standard floating point representation
 		set n 4
-		if { "$tcl_platform(byteOrder)" == "littleEndian" } {
+		if { $tcl_platform(byteOrder) == "littleEndian" } {
 		    binary scan [join [lrange $data 0 3] ""] "f" x
 		} else {
 		    set id ""
@@ -1239,7 +1328,7 @@ proc UnPackData {data types} {
 		# this only works with machines following the
 		#  IEEE standard floating point representation
 		set n 8
-		if { "$tcl_platform(byteOrder)" == "littleEndian" } {
+		if { $tcl_platform(byteOrder) == "littleEndian" } {
 		    binary scan [join [lrange $data 0 7] ""] "d" x
 		} else {
 		    set id ""
@@ -1290,7 +1379,7 @@ proc UnPackData {data types} {
 		regsub starray= $t "" ets
 		set ts [split $ets ","]
 		set x ""
-		while { "$data" != "" } {
+		while { $data != "" } {
 		    lappend x [UnPackData $data $ts]
 		    set data $PacketDataRest
 		}
@@ -1302,7 +1391,7 @@ proc UnPackData {data types} {
 			     11930464.7111111111111]
 		set lo [expr [UnPackData [lrange $data 4 7] long]/ \
 			     11930464.7111111111111]
-		set x "$la $lo"
+		set x [list $la $lo]
 		set n 8
 	    }
 	    radian {
@@ -1311,7 +1400,7 @@ proc UnPackData {data types} {
 			     57.29577951308232087684]
 		set lo [expr [UnPackData [lrange $data 8 15] double]* \
                              57.29577951308232087684]
-		set x "$la $lo"
+		set x [list $la $lo]
 		set n 16
 	    }
 	    unused=* {
@@ -1385,7 +1474,7 @@ proc DataToStr {types vals} {
 		# this only works with machines following the
 		#  IEEE standard floating point representation
 		set s [binary format "f" $v]
-		if { "$tcl_platform(byteOrder)" != "littleEndian" } {
+		if { $tcl_platform(byteOrder) != "littleEndian" } {
 		    set l [split "$s" ""]
 		    set s ""
 		    foreach k "3 2 1 0" {
@@ -1398,7 +1487,7 @@ proc DataToStr {types vals} {
 		# this only works with machines following the
 		#  IEEE standard floating point representation
 		set s [binary format "d" $v]
-		if { "$tcl_platform(byteOrder)" != "littleEndian" } {
+		if { $tcl_platform(byteOrder) != "littleEndian" } {
 		    set l [split "$s" ""]
 		    set s ""
 		    foreach k "7 6 5 4 3 2 1 0" {
@@ -1438,12 +1527,12 @@ proc DataToStr {types vals} {
 	    semicircle {
 		set sla [expr round([lindex $v 0]*11930464.7111111111111)]
 		set slo [expr round([lindex $v 1]*11930464.7111111111111)]
-		append data [DataToStr "long long" "$sla $slo"]
+		append data [DataToStr "long long" [list $sla $slo]]
 	    }
 	    radian {
 		set rla [expr [lindex $v 0]/57.29577951308232087684]
 		set rlo [expr [lindex $v 1]/57.29577951308232087684]
-		append data [DataToStr "double double" "$rla $rlo"]
+		append data [DataToStr "double double" [list $rla $rlo]]
 	    }
 	    unused=* {
 		regsub unused= $t "" n
@@ -1500,8 +1589,8 @@ proc FromGarminColour {pid code} {
 
 proc EndInProt {pid data} {
     # deal with end of input protocol
-    #  $pid in {A000 A001 A010 A100 A200 A201 A300 A301 A500 A600 A700} (cf.
-    #    ProcInPacket) but {A010 A500} not supported here
+    #  $pid in {A000 A001 A010 A100 A200-201 A300-302 A500 A600 A650 A700 A906}
+    #    (cf. ProcInPacket) but {A010 A500 A600 A650 A700} not supported here
     # global $Request==get$wh  where $wh in {WP RT TR PosnData DtTmData}
     #                          (cf. SendData)
     #                         GPSOpResult is set to 0 on success or 1 on error
@@ -1546,7 +1635,7 @@ proc EndInProt {pid data} {
 	get* {
 	    regsub get $Request "" wh
 	    SetCursor . watch
-	    if { "[lindex $data 0]" != "" } {
+	    if { [lindex $data 0] != "" } {
 		InData$wh $data
 		EndWPRenaming
 		set GPSOpResult 0
@@ -1572,13 +1661,14 @@ proc InDataWP {data} {
 
 proc InDataWPRT {data pid} {
     # add WPs data from receiver to database
-    # return list of names of WPs
+    # return list of names of WPs; it may be void if WPs were discarded because
+    #  of unacceptable data fields or because the user cancelled renaming
     global CurrPSPID GetDispl GetSet
 
     set wps ""
     foreach d [ConvWPData $data $CurrPSPID($pid)] {
 	set name [lindex $d 0] ; set ix [IndexNamed WP $name]
-	if { "$GetSet(WP)" == "" || [lsearch -exact $GetSet(WP) $ix] != -1 } {
+	if { $GetSet(WP) == "" || [lsearch -exact $GetSet(WP) $ix] != -1 } {
 	    if { ! [CheckName Ignore $name] } {
 		if { [set nname [AskForName $name]] == "" } { continue }
 		set d [AddOldNameToObs $d $name]
@@ -1595,16 +1685,17 @@ proc InDataWPRT {data pid} {
 proc ConvWPData {data pid} {
     # convert WPs data got from receiver into list of lists suitable for
     #  use with SetItem
+    # result may be an empty list
     global PositionFormat CREATIONDATE DATAFOR
 
     set r ""
     set fs $DATAFOR($pid,ns) ; set ps $DATAFOR($pid,ps)
     if { $CREATIONDATE } {
-	set all [linsert $fs 0 Datum Date Hidden]
-	set cnsts [list "WGS 84" [Now]]
+	set all [linsert $fs 0 Datum PFrmt Date Hidden]
+	set cnsts [list "WGS 84" "" [Now]]
     } else {
-	set all [linsert $fs 0 Datum Hidden]
-	set cnsts [list "WGS 84"]
+	set all [linsert $fs 0 Datum PFrmt Hidden]
+	set cnsts [list "WGS 84" ""]
     }
     foreach d $data {
 	set vs $cnsts
@@ -1613,14 +1704,23 @@ proc ConvWPData {data pid} {
 	    set v [lindex $d $p]
 	    switch $f {
 		Posn {
-		    set v [CreatePos [lindex $v 0] [lindex $v 1] \
-			    $PositionFormat "WGS 84"]
+		    foreach "latd longd" $v { break }
+		    if { $pid == "D110" && abs($latd) > 0x40000000 } {
+			# discard
+			continue
+		    }
+		    foreach "v pfmt datum" \
+			[FormatPosition $latd $longd "WGS 84" \
+			     $PositionFormat "" DDD] {
+			    break
+		    }
+		    set vs [lreplace $vs 0 1 $datum $pfmt]
 		}
 		Symbol {
 		    set v [NameForCodeOf SYMBOL $v]
 		}
 		DispOpt {
-		    if { "$pid" == "D109" } {
+		    if { $pid == "D109" || $pid == "D110" } {
 			# colour has been dealt with by HiddenGet
 			set v [expr $v>>5]
 		    }
@@ -1628,7 +1728,7 @@ proc ConvWPData {data pid} {
 		}
 		Alt {
 		    switch $pid {
-			D108 -  D109 {
+			D108 -  D109 -  D110 {
 			    if { $v > 1e10 } { set v "" }
 			}
 			D150 -  D151 -  D152 -  D154 -  D155 {
@@ -1636,7 +1736,7 @@ proc ConvWPData {data pid} {
 			    set k [lsearch -glob $hvs G*:class=*]
 			    if { $k != -1 && \
 				    "G${pid}:class=[HiddenCode int 0]" == \
-				    "[lindex $hvs $k]" } {
+				    [lindex $hvs $k] } {
 				set v ""
 			    }
 			}
@@ -1663,17 +1763,17 @@ proc InDataRT {data} {
 	set fs "IdNumber Commt Stages WPoints"
     }
     set hpid $CurrPSPID(RTHeader)
-    while { "$data" != "" } {
+    while { $data != "" } {
 	set r [ConvRTHeaderData [lindex $data 0] $hpid]
 	set id [lindex $r 0] ; set ix [IndexNamed RT $id]
-	if { "$GetSet(RT)" == "" || [lsearch -exact $GetSet(RT) $ix] != -1 } {
+	if { $GetSet(RT) == "" || [lsearch -exact $GetSet(RT) $ix] != -1 } {
 	    set GetSet(WP) ""
 	    set wpd [lindex $data 1]
 	    if { ! $nolk } {
 		set wps "" ; set sts ""
-		while { "$wpd" != "" } {
+		while { $wpd != "" } {
 		    lappend wps [lindex $wpd 0]
-		    if { "[set sd [lindex $wpd 1]]" != "" } {
+		    if { [set sd [lindex $wpd 1]] != "" } {
 			lappend sts [ConvRSData $sd $CurrPSPID(RTLinkData)]
 		    }
 		    set wpd [lreplace $wpd 0 1]
@@ -1683,7 +1783,8 @@ proc InDataRT {data} {
 	    }
 	    set wps [InDataWPRT $wpd RTWPData]
 	    # $wps can be void if the user cancelled replacement of names
-	    if { "$wps" != "" } {
+	    #  or $wps were discarded
+	    if { $wps != "" } {
 		lappend r $wps
 		set d [FormData RT $fs $r]
 		StoreRT $ix $id $d $wps $GetDispl
@@ -1735,28 +1836,28 @@ proc InDataTR {data} {
 	    # no support for segments
 	    foreach p $data {
 		set vals [lindex $p 1]
-		if { "[lindex $p 0]" != "data" } {
+		if { [lindex $p 0] != "data" } {
 		    Log "IDTR> got a non-data pair: [lindex $p 0]"
 		    return
 		}
 		set tpp [ConvTPData $vals $pidd]
-		if { [lindex $tpp 0] == 1 && "$tps" != "" } {
+		if { [lindex $tpp 0] == 1 && $tps != "" } {
 		    lappend trs [list "" $tps ""]
 		    set tps ""
 		}
 		lappend tps [lindex $tpp 1]
 	    }
-	    if { "$tps" != "" } {
+	    if { $tps != "" } {
 		lappend trs [list "" $tps ""]
 	    }
 	}
-	A301 {
+	A301 -  A302 {
 	    # segments are supported
 	    set pidh $CurrPSPID(TRHeader)
 	    set phdr "" ; set segsts "" ; set tpn 0
 	    foreach p $data {
 		set vals [lindex $p 1]
-		if { "[lindex $p 0]" != "header" } {
+		if { [lindex $p 0] != "header" } {
 		    set tpp [ConvTPData $vals $pidd]
 		    lappend tps [lindex $tpp 1]
 		    if { [lindex $tpp 0] && $tpn != 0 } {
@@ -1764,14 +1865,14 @@ proc InDataTR {data} {
 		    }
 		    incr tpn
 		} else {
-		    if { "$tps" != "" } {
-			if { "$phdr" == "" } {
+		    if { $tps != "" } {
+			if { $phdr == "" } {
 			    Log "IDTR> TPs without header"
 			    return
 			}
 			lappend trs [list $phdr $tps $segsts]
 			set tps ""
-		    } elseif { "$phdr" != "" } {
+		    } elseif { $phdr != "" } {
 			Log "IDTR> TR header without TPs; discarding"
 		    }
 		    if { [set phdr [ConvTRHeaderData $vals $pidh]] == -1 } {
@@ -1780,7 +1881,7 @@ proc InDataTR {data} {
 		    set tpn 0 ; set segsts ""
 		}
 	    }
-	    if { "$tps" != "" && "$phdr" != "" } {
+	    if { $tps != "" && $phdr != "" } {
 		lappend trs [list $phdr $tps $segsts]
 	    }
 	}
@@ -1805,11 +1906,14 @@ proc ConvTRHeaderData {data pid} {
     # return pair of lists with field names and values
 
     switch $pid {
-	D310 {
+	D310 -  D312 {
 	    set fs "Name Colour Hidden"
 	    set vs [list [lindex $data 2] \
-		    [FromGarminColour D310 [lindex $data 0]] \
-		    [HiddenGet D310 $data]]
+		    [FromGarminColour $pid [lindex $data 0]] \
+		    [HiddenGet $pid $data]]
+	}
+	D311 {
+	    set fs Name ; set vs [lindex $data 0]
 	}
     }
     return [list $fs $vs]
@@ -1821,31 +1925,130 @@ proc ConvTPData {data pid} {
     #  by FormData
 
     set fs "" ; set vs ""
+    # similar fields: position, date
+    set p [lindex $data 0] ; set d [lindex $data 1]
     switch $pid {
 	D300 {
-	    set p [lindex $data 0]
-	    set d [lindex $data 1]
 	    set new [lindex $data 2]
 	}
-	D301 {
-	    set p [lindex $data 0]
-	    set d [lindex $data 1]
+	D301 -  D302 {
 	    if { [set alt [lindex $data 2]] < 1e10 } {
 		lappend fs alt ; lappend vs $alt
 	    }
 	    if { [set dep [lindex $data 3]] < 1e10 } {
 		lappend fs depth ; lappend vs $dep
 	    }
-	    set new [lindex $data 4]
+	    if { $pid == "D302" } {
+		# discard temperature
+		set new [lindex $data 5]
+	    } else { set new [lindex $data 4] }
 	}
     }
-    set p [CreatePos [lindex $p 0] [lindex $p 1] DMS "WGS 84"]
+    if { [catch {set np [FormatLatLong [lindex $p 0] [lindex $p 1] DMS]}] } {
+	Log "ConvTPData> bad position: p=$p\ndata=$data"
+	GMMessage "Error getting TPs; please report to mig@ncc.up.pt"
+	return [list 1 {}]
+    }
     set dl [ConvGarminDate $d]
     lappend fs latd longd latDMS longDMS date secs
-    set tpdata [FormData TP $fs [concat $vs $p $dl]]
+    set tpdata [FormData TP $fs [concat $vs $np $dl]]
     return [list $new $tpdata]
 }
 
+proc InDataLAP {data} {
+    # add LAPs data from receiver to database
+    # create GRs with runs
+    #  $data is a list of lists of values
+    global CurrPSPID GetSet LAPName LAPTRIx LAPName TXT
+
+    set pidd $CurrPSPID(LAPData)
+    set ixs ""
+    switch $CurrPSPID(LAP) {
+	A906 {
+	    foreach d [ConvLAPData $data $pidd] {
+		set name [lindex $d 0] ; set ix [IndexNamed LAP $name]
+		if { $GetSet(LAP) == "" || \
+			 [lsearch -exact $GetSet(LAP) $ix] != -1 } {
+		    set ix [StoreLAP $ix $name $d]
+		    lappend ixs $ix
+		}
+	    }
+	}
+    }
+    # build runs: create a GR for each one
+    set run "" ; set n 0
+    foreach ix $ixs {
+	lappend run $LAPName($ix)
+	if { [set trix $LAPTRIx($ix)] < 255 } {
+	    set n "$TXT(run) $LAPName([lindex $run 0])"
+	    CreateGRFor "@$n@lapsrun" "" [list [list LAP $run]]
+	    set run ""
+	}
+    }
+    if { $run != "" && $trix == 255 } {
+	CreateGRFor lapsrun "" [list [list LAP $run]]
+    }
+    return
+}
+
+proc ConvLAPData {data pid} {
+    # convert LAPs data got from receiver into list of lists suitable for
+    #  use with SetItem
+    global PositionFormat DATAFOR
+
+    set r ""
+    set fs $DATAFOR($pid,ns) ; set ps $DATAFOR($pid,ps)
+    set all [linsert $fs 0 Datum PFrmt Name]
+    set cnsts [list "WGS 84" $PositionFormat ""]
+    foreach d $data {
+	set vs $cnsts
+	set ix 0
+	foreach f $fs p $ps {
+	    set v [lindex $d $p]
+	    switch -glob $f {
+		Start {
+		    set v [ConvGarminDate $v]
+		    set name [lindex $v 0]
+		}
+		Dur {
+		    set v [FormatTime [expr $v*0.01] 2]
+		}
+		*Posn {
+		    foreach "latd longd" $v { break }
+		    if { abs($latd) > 90 || abs($longd) > 180 } {
+			set v ""
+			set $f ""
+		    } else {
+			foreach "v pfmt datum" \
+			    [FormatPosition $latd $longd "WGS 84" \
+				 $PositionFormat "" DDD] { break }
+			# BegPosn and EndPosn must have same format and datum
+			set $f [list $pfmt $datum $latd $longd $ix]
+		    }
+		}
+	    }
+	    lappend vs $v
+	}
+	incr ix
+	if { $BegPosn != "" || $EndPosn != "" } {
+	    if { $BegPosn != "" && $EndPosn != "" && \
+		     [lrange $BegPosn 0 1] != [lrange $EndPosn 0 1] } {
+		# incompatible position format or datum: use DDD for both
+		foreach k [list $BegPosn $EndPosn] {
+		    set i [lindex $k 4]
+		    set vs [lreplace $vs $i $i \
+				[eval FormatLatLong [lrange $k 2 3] DDD]]
+		}
+		set pfmt DDD ; set datum "WGS 84"
+	    }
+	    set vs [lreplace $vs 0 1 $datum $pfmt]
+	}
+	set vs [lreplace $vs 2 2 $name]
+	lappend r [FormData LAP $all $vs]
+    }
+    return $r
+}
+
 proc ConvGarminDate {gd} {
     # converts Garmin date (seconds since 1990.01.01 00:00:00) into list
     #  with date in current format and seconds since beginning of $YEAR0,
@@ -1908,6 +2111,8 @@ proc InDataPVT {data pid} {
 	    foreach "alt epe eph epv fix tow posn vlong vlat valt mslh \
 		       lsecs wnds" \
 		    $data { break }
+	    # some old receivers may use a fix value that is 1 more than this
+	    #  (Garmin specs 1 rev A); that is not dealt with here!!!
 	    if { $fix < 2 } { return "" }
 	    if { $fix%2 == 0 } {
 		set alt "_" ; set valt "_"
@@ -2058,7 +2263,7 @@ proc HiddenFindVal {pid name hidden} {
 	    set i [string first : $hh]
 	    set h [string range $hh [expr $i+1] end]
 	    if { [set i [string first = $h]] > 0 } {
-		if { "[string range $h 0 [expr $i-1]]" == "$name" } {
+		if { [string range $h 0 [expr $i-1]] == $name } {
 		    set hd [HiddenDecode $type \
 			[string range $h [expr $i+1] end]]
 		    if { [lindex $hd 0] } {
@@ -2245,9 +2450,9 @@ proc HiddenCode {type val} {
     set chars [PackData [list $type [list $val]]]
     set r ""
     set shift 0
-    while { "$chars" != "" } {
+    while { $chars != "" } {
 	for { set c [lindex $chars 0] ; set n 1 } \
-		{ "$chars" != "" && "$c" == "[lindex $chars $n]" } \
+		{ $chars != "" && $c == [lindex $chars $n] } \
 		{ incr n } {
 	    continue
 	}
@@ -2324,12 +2529,12 @@ proc HiddenDecode {type coded} {
     set shift 0 ; set x 1
     array set dshift { 1 -33  2 128  3 95 }
     set lc [split $coded ""]
-    while { "$lc" != "" } {
+    while { $lc != "" } {
 	set info 1
 	switch -- [set c [lindex $lc 0]] {
 	    "|" {
 		set lc [lreplace $lc 0 0]
-		if { "$lc" == "" } {
+		if { $lc == "" } {
 		    Log "HD> nothing after |"
 		    return 0
 		}
@@ -2357,7 +2562,7 @@ proc HiddenDecode {type coded} {
 		    }
 		    "R" {
 			set lc [lreplace $lc 0 0]
-			if { "$lc" == "" } {
+			if { $lc == "" } {
 			    Log "HD> nothing after |R"
 			    return 0
 			}
@@ -2413,7 +2618,7 @@ proc InitGivenProtocols {data} {
     # return 0 on error
     global CurrPSPID PROTTAG PROTCAT PROTREQ PSPROTOCOLS PSPDEF PID RPID \
 	    PSCMDDEF CMD MyProdId MyProdDescr MyProdVersion NotLogging \
-	    USERDIR PRTCLDEF MESS VERSION PRTNUMID PRTHNID
+	    USERDIR PRTCLDEF MESS VERSION
 
     set npf [file join $USERDIR $MyProdId.prt]
     if { $NotLogging && [catch {set PRTCLDEF($MyProdId)}] && \
@@ -2524,7 +2729,7 @@ proc InitGivenProtocols {data} {
 	Log "IGP> aborting due to previous error(s)"
 	return 0
     }
-    if { "$CurrPSPID(Link)" == "N/A" || "$CurrPSPID(DevCmd)" == "N/A" } {
+    if { $CurrPSPID(Link) == "N/A" || $CurrPSPID(DevCmd) == "N/A" } {
 	if { $outpfile } {
 	    puts $pfile "Error: no Link or DevCmd protocol(s): $odata"
 	    close $pfile
@@ -2546,10 +2751,7 @@ proc InitGivenProtocols {data} {
 	GMMessage $pfmess
     }
     Log "IGP> done"
-    if { [lsearch -exact $PRTNUMID $CurrPSPID(RTHeader)] != -1 } {
-	# RT needs numeric id
-	set PRTHNID 1
-    } else { set PRTHNID 0 }
+    RecAdjustToProtocols
     return 1
 }
 
@@ -2558,7 +2760,7 @@ proc InitProtocols {prodid} {
     #  from the default table
     #  $prodid is the product identifier
     global PRTCLDEF PSPROTOCOLS PSPID PSDIFF PSPDEF PSCMDDEF CurrPSPID \
-	    CMD PID RPID PROTREQ PRTNUMID PRTHNID
+	    CMD PID RPID PROTREQ
 
     switch -glob $PRTCLDEF($prodid) {
 	array {
@@ -2587,17 +2789,47 @@ proc InitProtocols {prodid} {
 	    }
 	}
     }
-    if { [lsearch -exact $PRTNUMID $CurrPSPID(RTHeader)] != -1 } {
-	# RT needs numeric id
-	set PRTHNID 1
-    } else { set PRTHNID 0 }
+    RecAdjustToProtocols
+    return
+}
+
+proc RecAdjustToProtocols {} {
+    # adjust receiver parameters according to its protocols
+    global CurrPSPID RECTYPES PRTNUMID PRTHNID PTRNUMID PTRHNID
+
+    # $RECTYPES does not contain LAP if $SUPORTLAPS is not set
+    foreach t $RECTYPES {
+	# there are models without RTs, TRs or LAPs
+	#  $CurrPSPID(LAP) may not be defined
+	if { $t != "GR" && \
+		 ( [catch {set p $CurrPSPID($t)}] || $p == "N/A" ) } {
+	    set RECTYPES [Delete $RECTYPES $t]
+	} else {
+	    switch $t {
+		RT {
+		    if { ! [catch {set p $CurrPSPID(RTHeader)}] && \
+			     [lsearch -exact $PRTNUMID $p] != -1 } {
+			# RT needs numeric id
+			set PRTHNID 1
+		    } else { set PRTHNID 0 }
+		}
+		TR {
+		    if { ! [catch {set p $CurrPSPID(TRHeader)}] && \
+			     [lsearch -exact $PTRNUMID $p] != -1 } {
+			# TR needs numeric id
+			set PTRHNID 1
+		    } else { set PTRHNID 0 }
+		}
+	    }
+	}
+    }
     return
 }
 
 proc NoProtCapability {} {
     # receiver does not support the Protocol Capability Protocol
     #  so the default definitions should be used
-    global MyProdId PRTCLDEF PRTNUMID PRTHNID CurrPSPID
+    global MyProdId PRTCLDEF
 
     Log "NPC> no Protocol Capability protocol"
     if { [catch {set PRTCLDEF($MyProdId)}] } {
@@ -2605,10 +2837,6 @@ proc NoProtCapability {} {
 	return
     }
     InitProtocols $MyProdId
-    if { [lsearch -exact $PRTNUMID $CurrPSPID(RTHeader)] != -1 } {
-	# RT needs numeric id
-	set PRTHNID 1
-    } else { set PRTHNID 0 }
     EndConnCheck defprots
     return
 }
@@ -2646,7 +2874,7 @@ proc DecodeSimpleText {line} {
     # assume EPH and altitude to be given as integers
     global SimpleTextBegs SimpleTextEnds SimpleTextPStatus
 
-    if { "[string index $line 0]" != "@" } { return 0 }
+    if { [string index $line 0] != "@" } { return 0 }
     foreach f "y m d h mn s hlat dlat mlat hlong dlong mlong pst eph as alt \
 	       hvlong vlong hvlat vlat hvalt valt" \
 	    type "int int int int int int NS int int/1000 EW int int/1000 \
@@ -2711,6 +2939,24 @@ proc DecodeSimpleText {line} {
 
 ##### GPSMan interface
 
+proc StartGPS {} {
+    global NoGarmin
+
+    ResetAutoNumberRT
+    set NoGarmin 1
+    return
+}
+
+proc ResetAutoNumberRT {} {
+    # reset counter for renumbering RTs sent to receiver
+    # based on DJG contribution
+    global RTNextNumber RTSendNumber
+
+    set RTNextNumber 1
+    catch {unset RTSendNumber}
+    return
+}
+
 proc GPSChangeProtocol {prot} {
     # change current protocol
     #  $prot in {garmin, nmea, stext, simul} (for Garmin)
@@ -2718,13 +2964,13 @@ proc GPSChangeProtocol {prot} {
     global GPSProtocol GPSProtocolExt GPSState TXT NoGarmin SRLFILE Eof \
 	    RealTimeLogOn
 
-    if { "$GPSProtocol" == "$prot" } { return }
+    if { $GPSProtocol == $prot } { return }
     if { ! $NoGarmin } {
 	# change cannot be to garmin
 	Log "GCP> changing to protocol $prot"
 	set NoGarmin 1
     }
-    if { "$GPSState" == "online" } {
+    if { $GPSState == "online" } {
 	set Eof 1
 	set GPSState offline
 	close $SRLFILE
@@ -2753,7 +2999,7 @@ proc AbortComm {args} {
 	catch { after cancel $j }
     }
     ResetSerial
-    if { "$args" != "" } { GMMessage $MESS($args) }
+    if { $args != "" } { GMMessage $MESS($args) }
     set Jobs ""
     switch -glob $Request {
 	get* {
@@ -2793,7 +3039,7 @@ proc GarminStartPVT {} {
 	return 0
     }
     if { [catch {set CurrPSPID(PVTData)}] || \
-	    "$CurrPSPID(PVTData)" == "N/A" } {
+	    $CurrPSPID(PVTData) == "N/A" } {
 	GMMessage $MESS(rltmnotsupp)
 	return 0
     }
@@ -2836,7 +3082,7 @@ proc StopLineProtocol {} {
 
     # make sure the read char proc called by fileevent stops
     set Eof 1
-    if { "$GPSState" == "online" } {
+    if { $GPSState == "online" } {
 	close $SRLFILE
     }
     DisableGPS
