@@ -17,12 +17,14 @@
 #      along with this program.
 #
 #  File: garmin_protocols.tcl
-#  Last change:  7 June 2004
+#  Last change:  18 October 2004
 #
 
 # description of Garmin protocols based on Garmin GPS Interface
 #  Specification Version 1 Rev. 2
-# updated for Garmin GPS Interface Specification Version 1 Rev. 3
+# updated for:
+#   Garmin GPS Interface Specification Version 1 Rev. 3
+#   Garmin GPS Interface Specification Version 1 Rev. A
 
    # ASCII codes (decimal)
 set DLE 16
@@ -47,18 +49,21 @@ array set PROTCAT {
     A201  RT
     A300  TR
     A301  TR
+    A302  TR
     A400  Prx
     A500  AL
     A600  DtTm
     A601  nospec
-    A650  nospec
+    A650  FlightBook
     A700  Posn
     A800  PVT
+    A802  nospec
     A900  nospec
     A901  nospec
     A902  nospec
     A903  nospec
     A904  nospec
+    A906  LAP
 }
 
    # protocol requirements
@@ -75,11 +80,12 @@ array set PROTREQ {
     A201  {RTHeader RTWPData RTLinkData}
     A300  TRData
     A301  {TRHeader TRData}
+    A302  {TRHeader TRData}
     A400  PrxWPData
     A500  ALData
     A600  DtTmData
     A601  D?
-    A650  D?
+    A650  ""
     A700  PosnData
     A800  PVTData
     A802  D?
@@ -88,6 +94,7 @@ array set PROTREQ {
     A902  ""
     A903  ""
     A904  ""
+    A906  LAPData
 }
 
    # Basic Link Protocol
@@ -104,17 +111,18 @@ set PSPROTOCOLS {Link DevCmd WP RT TR Prx AL DtTm Posn PVT}
 set PSPDEF(L001) {{CmdData 10} {XfrCmpl 12} {DtTmData 14} {PosnData 17}
                   {PrxWPData 19} {Records 27} {RTHeader 29} {RTWPData 30}
                   {ALData 31} {TRData 34} {WPData 35} {PVTData 51}
-	          {RTLinkData 98} {TRHeader 99}}
+                  {RTLinkData 98} {TRHeader 99} {FlightBook_Record 134}
+                  {LAPData 149}}
 set PSPDEF(L002) {{ALData 4} {CmdData 11} {XfrCmpl 12} {DtTmData 20}
-                  {PosnData 24} {Records 35} {RTHeader 37} {RTWPData 39}
-                  {WPData 43}}
+                  {PosnData 24} {PrxWPData 27} {Records 35} {RTHeader 37}
+                  {RTWPData 39} {WPData 43}}
 
     # definition of product specific device commands
 set PSCMDDEF(A010) {{Abort 0} {XfrAL 1} {XfrPosnData 2} {XfrPrx 3} {XfrRT 4}
                   {XfrDtTmData 5} {XfrTR 6} {XfrWP 7} {TurnOff 8} {StartPVT 49}
-                  {StopPVT 50}}
-set PSCMDDEF(A011) {{Abort 0} {XfrAL 1} {XfrRT 8} {XfrDtTmData 20} {XfrWP 21}
-                  {TurnOff 26}}
+                  {StopPVT 50} {XfrFlightBook 92} {XfrLAP 117}}
+set PSCMDDEF(A011) {{Abort 0} {XfrAL 1} {XfrRT 8} {XfrPrx 17} {XfrDtTmData 20}
+                  {XfrWP 21} {TurnOff 26}}
 
     # size in bytes of data types used in unions
     # unions are only used if there is a single data item and sizes of
@@ -154,7 +162,11 @@ array set PDTYPE {
     D108  {byte byte byte byte int bytes=18 semicircle float float float
            charray=2 charray=2 string string string string string string}
     D109  {byte byte byte byte int bytes=18 semicircle float float float
-           charray=2 charray=2 long string string string string string string}
+           charray=2 charray=2 longword string string string string string
+	   string}
+    D110  {byte byte byte byte int bytes=18 semicircle float float float
+           charray=2 charray=2 longword float longword int string string string
+	   string string string}
     D150  {charray=6 charray=2 byte semicircle int charray=24 charray=2
 	   charray=30 charray=40}
     D151  {charray=6 semicircle unused=4 charray=40 float charray=30
@@ -171,41 +183,55 @@ array set PDTYPE {
     D210  {word bytes=18 string}
     D300  {semicircle longword boolean}
     D301  {semicircle longword float float boolean}
+    D302  {semicircle longword float float float boolean}
     D310  {boolean byte string}
+    D311  word
+    D312  {boolean byte string}
     D400  {charray=6 semicircle unused=4 charray=40 float}
     D403  {charray=6 semicircle unused=4 charray=40 byte byte float}
+    D450  {int charray=6 charray=2 byte semicircle int charray=24 charray=2
+	   charray=30 charray=40 float}
     D500  {int float float float float float float float float float float}
     D501  {int float float float float float float float float float float
            byte}
+    D550  {byte int float float float float float float float float float
+	   float}
+    D551  {byte int float float float float float float float float float
+	   float byte}
     D600  {byte byte word int byte byte}
     D601  nospec
-    D650  nospec
+    D650  {longword longword semicircle semicircle longword longword float
+           float float boolean string string string string string}
     D700  radian
     D800  {float float float float int double radian float float float float
            int long}
     D802  nospec
     D901  nospec
+    D906  {longword longword float semicircle semicircle word byte byte}
 }
 
-## RT Header protocols with a numeric identifier
+## RT/TR Header protocols with a numeric identifier
 set PRTNUMID {D200 D201}
+set PTRNUMID {D311}
 
     # product identifiers; new entries should also be added to recmodels.tcl
     #  numbers followed by a letter are defined here to support older
     #  software versions; not following Garmin specs in this
 array set PRODID {
+    "GPS 50" 7  
     "GPS 75" {13 23 42}
     "GPS 55" 14
     "GPS 55 AVD" 15
     "GPS 65" 18
-    "GPS 95 AVD" 22
+    "GPS 95 AVD" {22 36}
     "GPS 95" {24 35}
+    "GPS 85" 25
     "GPSMAP 205" {29 44}
     "GPSMAP 210" 29
     "GPSMAP 220" 29
     "GPS 40" {31 41}
     "GPS 45" {31 41}
-    "GPS 95 XL" {36 22}
+    "GPS 95 XL" 36
     "GPS 89" 39
     "GPS 38" 41
     "GPS 45 XL" 41
@@ -234,15 +260,21 @@ array set PRODID {
     "GPSMAP 130 Chinese" 76
     "GPSMAP 230 Chinese" 76
     "GPS 12" {77 87 96}
-    "GPS 12 XL" 77
+    "GPS 12 XL" {77 96}
     "GPS 12 XL - software < 3.01" 77a
+    "GPSMAP 215" 88
+    "GPSMAP 225" 88
     "GPSMAP 180" 89
-    "GPS 126" {95 100}
-    "GPS 128" {95 100}
+    "GPS 126" 95
+    "GPS 128" 95
     "GPS 48" 96
+    "GPS 126 Chinese" 100
+    "GPS 128 Chinese" 100
+    "GPS 12 Arabic" 103
     "GPS 12 XL Japanese" 105
     "GPS 12 XL Chinese" 106
     eMap   111
+    "GPS 92" 112
     "GPS 12CX" 116
     "GPS III Plus" 119
     "GPSMAP 295" 128
@@ -265,6 +297,7 @@ array set PRODID {
     "GPS 72" 247
     "Geko 201" 248
     "Geko 301" 256
+    "Forerunner" 282
     "GPSMap60C" 291
     "ForeTrex" 314
 }
@@ -280,6 +313,7 @@ array set PRODID {
     #            the other product and must not contain Link or DevCmd
 
 array set PNAME {
+     7  "GPS 50"
     13  "GPS 75"
     14  "GPS 55"
     15  "GPS 55 AVD"
@@ -287,10 +321,13 @@ array set PNAME {
     22  "GPS 95 AVD"
     23  "GPS 75"
     24  "GPS 95"
+    25  "GPS 85"
+    29a  "GPSMAP 205/210/220 - <4.0"
     29  "GPSMAP 205/210/220"
     31  "GPS 40/45"
     35  "GPS 95"
-    36  "GPS 95 XL"
+    36a  "GPS 95 AVD/XL - < 3.0"
+    36  "GPS 95 AVD/XL"
     39  "GPS 89"
     41  "GPS 38/40/45/45 XL"
     42  "GPS 75"
@@ -311,18 +348,23 @@ array set PNAME {
     73  "GPS II Plus"
     74  "GPS 120 XL"
     76  "GPSMAP 130 Chinese/230 Chinese"
-    77a "GPS 12 XL - software < 3.01"
+    77a "GPS 12 XL - < 3.01"
+    77b "GPS 12 XL - 3.01 < 3.50"
+    77c "GPS 12 XL - 3.50 < 3.61"
     77  "GPS 12/12 XL"
     87  "GPS 12"
+    88  "GPSMAP 215/225"
     89  "GPSMAP 180"
     95  "GPS 126/128"
-    96  "GPS 12/48"
+    96  "GPS 12/12 XL/48"
     97  "GPS II Plus"
     100  "GPS 126 Chinese/128 Chinese"
+    103  "GPS 12 Arabic"
     105  "GPS 12 XL Japanese"
     106  "GPS 12 XL Chinese"
     119  "GPS III Plus"
     111  eMap
+    112  "GPS 92"
     116  "GPS 12CX"
     128  "GPSMAP 295"
     129  "GPS 12 Map"
@@ -344,28 +386,33 @@ array set PNAME {
     247  "GPS 72"
     248  "Geko 201"
     256  "Geko 301"
+    282  "Forerunner"
     291  "GPSMap60C"
     314  "ForeTrex"
 }
 
 # the descriptions below are valid for all versions except:
-#  29 version >= 4.0
-#  36 version >= 3.0; for older versions see=22
-#  77 version >= 3.61; may work for some older versions
-#  77a defined for 77 version < 3.01
+#  29 version < 4.0
+#  29+ version >= 4.0
+#  36 version < 3.0
+#  36+ version >= 3.0
+#  77, 77+, 77++ versions < 3.01, < 3.50, < 3.61
+#  77+++  version >= 3.61
 #  111 version <= 2.7
-#  111a version > 2.7
+#  111+ version > 2.7
 
       # the pairs in the following entries have a comparison to be made
       #  to the receiver version and the identifier to be used if the
       #  comparison succeeds
 array set PRTCLVERSION {
-    36  {{{< 300} 22}}
-    77  {{{< 301} 77a}}
-    111 {{{> 270} 111a}}
+    29  {{{>= 400} 29+}}
+    36  {{{>= 300} 36+}}
+    77  {{{>=361} 77+++} {{>= 350} 77++} {{>= 301} 77+}}
+    111 {{{>= 270} 111+}}
 }
 
 array set PRTCLDEF {
+     7  diff
     13  diff
     14  diff
     15  diff
@@ -373,14 +420,17 @@ array set PRTCLDEF {
     22  diff
     23  see=13
     24  see=13
+    25  see=13
     29  diff
+    29+ diff
     31  see=59
     35  see=13
-    36  see=45
+    36  see=22
+    36+ see=45
     39  diff
     41  see=59
     42  see=13
-    44  diff
+    44  see=29
     45  diff
     47  see=59
     48  diff
@@ -398,23 +448,28 @@ array set PRTCLDEF {
     74  see=59
     76  see=49
     77  diff
-    77a diff
-    87  see=77
-    89  see=29
-    95  see=77
-    96  see=77
+    77+ diff
+    77++ see=73
+    77+++  see=77+
+    87  see=77+
+    88  see=49
+    89  see=29+
+    95  see=77+
+    96  see=77+
     97  see=73
-    100  see=77
-    105  see=77
-    106  see=77
+    100  see=77+
+    103  see=77+
+    105  see=77+
+    106  see=77+
     111  array
-    111a diff
+    111+ diff
+    112  diff
     116  array
     119  array
     128  see=111
     129  array
     130  diff
-    138  see=77
+    138  see=77+
     141  see=130
     145  array
     151  array
@@ -426,11 +481,12 @@ array set PRTCLDEF {
     177  array
     179  see=130
     194  see=177
-    197  see=177
-    209  see=177
+    197  diff
+    209  see=197
     247  see=116
     248  array
     256  diff
+    282  array
     291  array
     314  array
 }
@@ -628,6 +684,27 @@ array set PSPID {
     248,PVT A800
     248,PVTData D800
 
+    282,Link L001
+    282,DevCmd A010
+    282,WP A100
+    282,WPData D108
+    282,RT N/A
+    282,TR A302
+    282,TRHeader D311
+    282,TRData D301
+    282,Prx  N/A
+    282,AL A500
+    282,ALData D501
+    282,DtTm A600
+    282,DtTmData D600
+    282,Posn A700
+    282,PosnData D700
+    282,PVT A800
+    282,PVTData D800
+    282,LAP A906
+    282,LAPData D906
+    282,nospec A802+A903
+
     291,Link L001
     291,DevCmd A010
     291,WP A100
@@ -649,7 +726,7 @@ array set PSPID {
     291,PosnData D700
     291,PVT A800
     291,PVTData D800
-    291,nospec A601+D601+A900+A902+A903
+    291,nospec A601+D601+A900+A902+A903+A904
 
     314,Link L001
     314,DevCmd A010
@@ -662,6 +739,7 @@ array set PSPID {
     314,TR A301
     314,TRHeader D310
     314,TRData D301
+    314,Prx N/A
     314,AL A500
     314,ALData D501
     314,DtTm A600
@@ -675,29 +753,34 @@ array set PSPID {
 
           # must not differ on the Link and DevCmd protocols
 array set PSDIFF {
-    13  {59 {Prx A400} {PrxData D400}}
-    14  {13 {RTHeader D200}}
+     7  {59 {RTHeader D200} {TR N/A}}
+    13  {59 {RTHeader D200} {Prx A400} {PrxData D400}}
+    14  {59 {RTHeader D200} {TR N/A} {Prx A400} {PrxData D400}}
     15  {14 {WPData D151} {RTWPData D151} {PrxData D151}}
-    22  {45 {Prx A400} {PrxData D152}}
-    29  {59 {WPData D102} {RTWPData D102} {Prx A400} {PrxData D102}}
+    22  {13 {WPData D152} {RTWPData D152} {PrxData D152}}
+    29  {59 {WPData D101} {RTWPData D101} {Prx A400} {PrxData D101}}
+    29+ {29 {WPData D102} {RTWPData D102} {PrxData D102}}
     39  {59 {WPData D151} {RTWPData D151}}
-    44  {59 {WPData D101} {RTWPData D101} {Prx A400} {PrxData D101}}
     45  {59 {WPData D152} {RTWPData D152}}
     48  {59 {WPData D154} {RTWPData D154} {ALData D501}}
     49  {29 {ALData D501}}
     50  {45 {ALData D501}}
     71  {59 {WPData D155} {RTWPData D155} {ALData D501}}
-    72  {59 {WPData D104} {RTWPData D104} {ALData D501}}
-    73  {59 {WPData D103} {RTWPData D103} {ALData D501}}
-    77  {73 {Prx A400} {PrxData D403}}
-    77a {59 {ALData D501}}
-    130 {111 {RTHeader D202}}
-    111a {111 {WPData D109} {RTWPData D109} {PrxWPData D109}}
-    155 {145 {nospec A900+A902+A903+A904}}
-    256 {248 {nospec A802}}
+    72  {71 {WPData D104} {RTWPData D104}}
+    73  {71 {WPData D103} {RTWPData D103}}
+    77  {59 {Prx A400} {PrxData D400} {ALData D501}}
+    77+ {73 {Prx A400} {PrxData D403}}
+   130  {111 {RTHeader D202}}
+   111+ {111 {WPData D109} {RTWPData D109} {PrxWPData D109}}
+   112  {59 {WPData D152} {RTWPData D152} {ALData D501}}
+   155  {145 {nospec A900+A902+A903+A904}}
+   197  {177 {Prx N/A}}
+   256  {248 {nospec A802}}
 }
 
 ## descriptions, positions in Garmin structures of GPSMan data
+#   normally the suffixes of each data array name, but see
+#   the corresponding proc Conv${wh}Data (garmin.tcl)
 
 array set DATAFOR {
     D100,ns {Name Posn Commt}
@@ -730,6 +813,9 @@ array set DATAFOR {
     D109,ns {DispOpt Symbol Posn Alt Name Commt}
     D109,ps {2 4 6 7 13 14}
 
+    D110,ns {DispOpt Symbol Posn Alt Name Commt}
+    D110,ps {2 4 6 7 16 17}
+
     D150,ns {Name Posn Alt Commt}
     D150,ps {0 3 4 8}
 
@@ -747,6 +833,15 @@ array set DATAFOR {
 
     D310,ns {Colour Name}
     D310,ps {0 2}
+
+    D311,ns Name
+    D311,ps 0
+
+    D312,ns {Colour Name}
+    D312,ps {0 2}
+
+    D906,ns {Start Dur Dist BegPosn EndPosn Cals TRIx}
+    D906,ps {0 1 2 3 4 5 6}
 }
 
 ## colours: names (as in rgb.tcl) ordered as in a C enumeration according to
@@ -756,6 +851,10 @@ array set GARMINCOLOURS {
     D310 {black darkred darkgreen darkyellow darkblue darkmagenta darkcyan
           lightgray darkgray red green yellow blue magenta cyan white}
     D310,dvar DEFTTRCOLOUR
+    D312 {black darkred darkgreen darkyellow darkblue darkmagenta darkcyan
+          lightgray darkgray red green yellow blue magenta cyan white
+	  transparent}
+    D312,dvar DEFTTRCOLOUR
 }
 
 ## hidden attributes:
@@ -783,12 +882,12 @@ array set HIDDENFOR {
     D107,fm {{colour enum {default red green blue}}}
 
     D108,ns {class colour attrs subclass depth state country facility
-    city addr int_road}
+	city addr int_road}
     D108,ts {byte byte byte bytes=18 float charray=2 charray=2
-    string string string string}
+	string string string string}
     D108,vs {0 255 96
-    "0 0 0 0 0 0 255 255 255 255 255 255 255 255 255 255 255 255"
-    1.0e25 "" "" "" "" "" ""}
+	"0 0 0 0 0 0 255 255 255 255 255 255 255 255 255 255 255 255"
+	1.0e25 "" "" "" "" "" ""}
     D108,ps {0 1 3 5 8 10 11 14 15 16 17}
     D108,cs {
 	{ if { $class==0 || $class>0x46 } { lappend undef city facility } }
@@ -801,19 +900,18 @@ array set HIDDENFOR {
 			    {0x44 avn_airp_rway} {0x45 avn_airp_int}
 			    {0x46 avn_airp_NDB} {0x80 map_pt}
 			    {0x81 map_area} {0x82 map_int}
-			    {0x83 map_addr} {0x84 map_label}
-			    {0x85 map_line}}}
+			    {0x83 map_addr} {0x84 map_line}}}
 	     {colour enumd {black dark_red dark_green dark_yellow dark_blue
 			    dark_magenta dark_cyan light_gray dark_gray
 			    red green yellow blue magenta cyan white default}}}
 
     D109,ns {dtyp class colour attrs subclass depth state country ete facility
-    city addr int_road}
+	city addr int_road}
     D109,ts {byte byte byte byte bytes=18 float charray=2 charray=2
-    long string string string string}
+	longword string string string string}
     D109,vs {1 0 0x1f 0x70
-    "0 0 0 0 0 0 255 255 255 255 255 255 255 255 255 255 255 255"
-    1.0e25 "" "" 0xffffffff "" "" "" ""}
+	"0 0 0 0 0 0 255 255 255 255 255 255 255 255 255 255 255 255"
+	1.0e25 "" "" 0xffffffff "" "" "" ""}
     D109,ps {0 1 2 3 5 8 10 11 12 15 16 17 18}
     D109,cs {
 	{ if { $class==0 || $class>0x46 } { lappend undef city facility } }
@@ -827,12 +925,37 @@ array set HIDDENFOR {
 			    {0x44 avn_airp_rway} {0x45 avn_airp_int}
 			    {0x46 avn_airp_NDB} {0x80 map_pt}
 			    {0x81 map_area} {0x82 map_int}
-			    {0x83 map_addr} {0x84 map_label}
-			    {0x85 map_line}}}
+			    {0x83 map_addr} {0x84 map_line}}}
 	     {colour enumd {black dark_red dark_green dark_yellow dark_blue
 			    dark_magenta dark_cyan light_gray dark_gray
 			    red green yellow blue magenta cyan white default}}}
 
+    D110,ns {dtyp class colour attrs subclass depth state country ete temp
+	     time cat facility city addr int_road}
+    D110,ts {byte byte byte byte bytes=18 float charray=2 charray=2
+	     longword float longword int string string string string}
+    D110,vs {1 0 0 0x80
+	"0 0 0 0 0 0 255 255 255 255 255 255 255 255 255 255 255 255"
+	1.0e25 "" "" 0xffffffff 1.0e25 0xfffffffff 0 "" "" "" ""}
+    D110,ps {0 1 2 3 5 8 10 11 12 13 14 15 18 19 20 21}
+    D110,cs {
+	{ if { $class==0 || $class>0x46 } { lappend undef city facility } }
+	{ if { $class<0x80 } { lappend undef subclass } }
+	{ if { $class!=0x83 } { lappend undef addr } }
+	{ if { $class!=0x82 } { lappend undef int_road } }
+    }
+    D110,pp {{colour &0x1f}}
+    D110,fm {{class envals {{0 user} {0x40 avn_airport} {0x41 avn_inters}
+                            {0x42 avn_NDB} {0x43 avn_VOR}
+			    {0x44 avn_airp_rway} {0x45 avn_airp_int}
+			    {0x46 avn_airp_NDB} {0x80 map_pt}
+			    {0x81 map_area} {0x82 map_int}
+			    {0x83 map_addr} {0x84 map_line}}}
+	     {colour enumd {black dark_red dark_green dark_yellow dark_blue
+			    dark_magenta dark_cyan light_gray dark_gray
+			    red green yellow blue magenta cyan white
+		            transparent}}}
+
     D150,ns {country class city state facility}
     D150,ts {charray=2 byte charray=24 charray=2 charray=30}
     D150,vs {"" 4 "" "" ""}
@@ -896,6 +1019,13 @@ array set HIDDENFOR {
     D310,ps {1}
     D310,cs {}
     D310,fm {{display enumd {no  yes}}}
+
+    D312,ns {display}
+    D312,ts {boolean}
+    D312,vs {0}
+    D312,ps {1}
+    D312,cs {}
+    D312,fm {{display enumd {no  yes}}}
 }
 
 # no longer used hidden attributes
