@@ -17,7 +17,7 @@
 #      along with this program.
 #
 #  File: garmin_symbols.tcl
-#  Last change:  15 March 2004
+#  Last change:  24 September 2004
 #
 
 # symbols used by Magellan
@@ -33,7 +33,7 @@ set MAG_SYMTAB {WP_dot square_green house avn_vortac airport amusement_park
 
 
 # symbol and display option codes from Garmin GPS Interface Specification
-#  Version 1 Rev. 3, plus a few others used in new receivers
+#  Version 1 Rev. 3 and Rev. A
 
 proc GarminStdSymbols {} {
     # set codes for set of standard Garmin symbols
@@ -108,7 +108,20 @@ proc GarminStdSymbols {} {
 	exit             177
 	flag             178
 	circle_x         179
-	house_2          181
+	open_24hr        180
+	fhs_facility     181
+	bot_cond         182
+	tide_pred_stn    183
+        anchor_prohib    184
+	beacon           185
+	coast_guard      186
+	reef             187
+	weedbed          188
+	dropoff          189
+	dock             190
+	marina           191
+	bait_tackle      192
+	stump            193
 	is_highway      8192
 	us_highway      8193
 	st_highway      8194
@@ -172,6 +185,27 @@ proc GarminStdSymbols {} {
 	border          8254
 	geocache        8255
 	geocache_fnd    8256
+	cntct_smiley    8257
+	cntct_ball_cap  8258
+	cntct_big_ears  8259
+	cntct_spike     8260
+	cntct_goatee    8261
+	cntct_afro      8262
+	cntct_dreads    8263
+	cntct_female1   8264
+	cntct_female2   8265
+	cntct_female3   8266
+	cntct_ranger    8267
+	cntct_kung_fu   8268
+	cntct_sumo      8269
+	cntct_pirate    8270
+	cntct_biker     8271
+	cntct_alien     8272
+	cntct_bug       8273
+	cntct_cat       8274
+	cntct_dog       8275
+	cntct_pig       8276
+	hydrant         8282
 	flag_pin_blue   8284
 	flag_pin_green  8285
 	flag_pin_red    8286
@@ -182,6 +216,62 @@ proc GarminStdSymbols {} {
 	box_green       8291
 	box_red         8292
 	biker           8293
+	circle_red      8294
+	circle_green    8295
+	circle_blue     8296
+	diamond_blue    8299
+	oval_red        8300
+	oval_green      8301
+	oval_blue       8302
+	rect_red        8303
+	rect_green      8304
+	rect_blue       8305
+	square_blue     8308
+	letter_a_red    8309
+	letter_b_red    8310
+	letter_c_red    8311
+	letter_d_red    8312
+	letter_a_green  8313
+	letter_c_green  8314
+	letter_b_green  8315
+	letter_d_green  8316
+	letter_a_blue   8317
+	letter_b_blue   8318
+	letter_c_blue   8319
+	letter_d_blue   8320
+	number_0_red    8321
+	number_1_red    8322
+	number_2_red    8323
+	number_3_red    8324
+	number_4_red    8325
+	number_5_red    8326
+	number_6_red    8327
+	number_7_red    8328
+	number_8_red    8329
+	number_9_red    8330
+	number_0_green  8331
+	number_1_green  8332
+	number_2_green  8333
+	number_3_green  8334
+	number_4_green  8335
+	number_5_green  8336
+	number_6_green  8337
+	number_7_green  8338
+	number_8_green  8339
+	number_9_green  8340
+	number_0_blue   8341
+	number_1_blue   8342
+	number_2_blue   8343
+	number_3_blue   8344
+	number_4_blue   8345
+	number_5_blue   8346
+	number_6_blue   8347
+	number_7_blue   8348
+	number_8_blue   8349
+	number_9_blue   8350
+	triangle_blue   8351
+	triangle_green  8352
+	triangle_red    8353
 	airport         16384
 	intersection    16385
 	avn_ndb         16386
