@@ -17,7 +17,7 @@
 #      along with this program.
 #
 #  File: gendials.tcl
-#  Last change:  20 May 2004
+#  Last change:  16 September 2004
 #
 # Includes contributions by
 #  - Brian Baulch (baulchb@onthenet.com.au) marked "BSB contribution"
@@ -26,9 +26,11 @@
 
   # modal dialogs
 
-proc GMMessage {mess} {
+proc GMMessage {mess args} {
     # create modal dialog for displaying message
-    #  single button: OK; binding: return
+    #  if $args=="wait"  return only when user acknowledges message
+    #      except in command-line mode or if using the slow op window
+    # single button: OK; binding: return
     global COLOUR EPOSX EPOSY TXT UNIX CMDLINE
 
     if { $CMDLINE} {
@@ -64,6 +66,13 @@ proc GMMessage {mess} {
     RaiseWindow .mess
     update idletasks
     grab .mess
+    if { $args == "wait" } {
+	while 1 {
+	    after 500
+	    update
+	    if { ! [winfo exists .mess] } { return }
+	}
+    }
     return
 }
 
@@ -363,9 +372,9 @@ proc SlowOpWindow {mess} {
     #  and returns 0  otherwise
     # any call within the loop to GMMessage will be diverted to this dialog
     # after the main loop there should be a call to proc SlowOpFinish
-    global SlowOp COLOUR MAPCOLOUR EPOSX EPOSY TXT CMDLINE
+    global SlowOp COLOUR MAPCOLOUR EPOSX EPOSY TXT CMDLINE USESLOWOPWINDOW
 
-    if { $CMDLINE } { return }
+    if { $CMDLINE || ! $USESLOWOPWINDOW } { return }
 
     if { [winfo exists .slowop] } {
 	.slowop.fr.title configure -text $mess
@@ -443,15 +452,14 @@ proc SlowOpFinish {mess} {
     #  $mess will be displayed if not empty
     # the dialog window will be closed silently if there were no messages,
     #  otherwise the Ok button is activated and the user must acknowledge it
-    global SlowOp TXT CMDLINE
+    global SlowOp TXT
 
-    if { $CMDLINE } { return }
-    if { ! $SlowOp(aborting) } { set SlowOp(status) $TXT(errwarn) }
-    set SlowOp(aborting) 0
     if { ! [winfo exists .slowop] } {
 	if { $mess != "" } { GMMessage $mess }
 	return
     }
+    if { ! $SlowOp(aborting) } { set SlowOp(status) $TXT(errwarn) }
+    set SlowOp(aborting) 0
     if { $mess != "" } { SlowOpMessage $mess }
     set txt .slowop.fr.frt.txt
     if { [$txt index end] == 2.0 } {
@@ -466,7 +474,7 @@ proc SlowOpFinish {mess} {
 }
 
 proc SlowOpAborted {} {
-    # to be called by application to test if the operation was aborted
+    # to be called by the application to test if the operation was aborted
     # if not a call to update is made to ensure that the window is usable
     # return 1 if yes
     global SlowOp TXT
@@ -915,4 +923,439 @@ proc GMUseEntries {w es} {
     return
 }
 
+### image listbox widget
+
+proc ImageListbox {act path args} {
+    # implements a new widget whose model is a listbox but has entries
+    #  with an image and possibly a text label
+    #  $act is the action to perform and determines $args
+    #    create  SIZE WIDTH EHEIGHT SELECTMODE ?SCROLLBAR?
+    #    insert  INDEX IMAGE TEXT ?TAGS?; return either index or -1 if entry
+    #      can not be inserted because IMAGE cannot be displayed and TEXT
+    #      is empty
+    #    delete  INDEX ?INDEX?
+    #    get     INDEX ?INDEX? ; return list of texts in entries
+    #    gettags INDEX ?INDEX? ; return list of tags in entries
+    #    selclr  INDEX ?INDEX? ; clear selected
+    #    selset  INDEX ?INDEX? ; set as selected (irrespective of SELECTMODE)
+    #    cursel  "" ; return list of indices of currently selected entries
+    #    getsel  "" ; return list of texts in currently selected entries
+    #    getseltags "" ; return list of lists each with the tags in currently
+    #      selected entries
+    #    seldel  "" ; delete selected entries
+    #    index   Y ; return index of entry at y-coordinate (inside listbox)
+    #    destroyall "" ; destroy all image listboxes under window $path
+    #  where
+    #     SELECTMODE is one of {single, extended}
+    #     EHEIGHT is the height for the entries in pixels (minimum used: 5)
+    #     INDEX is either a numeric index from 0 or "end"
+    #     TAGS is a list
+    # the widget should be packed or grided by caller after being created
+    # images that have more than EHEIGHT-4 in width or height are either
+    #  truncated to that size if they are of type photo, or not displayed
+    # information related to these widgets is stored in global array GMIBox
+    # auxiliary images are created but never deleted; their names can be
+    #  retrieved from GMIBox(img,*) entries
+    # bindings on entries:
+    #  <Button-1> deselects everything, selects entry
+    #  if SELECTMODE=="extended":
+    #    <Control-Button-1> toggles selection state of entry
+    #    <Shift-Button-1> selects range from last selected entry to entry
+    global GMIBox COLOUR
+
+    if { $act == "destroyall" } {
+	foreach n [array names GMIBox $path*,csize] {
+	    regsub {,csize$} $n "" lbox
+	    destroy $lbox
+	}
+	foreach n [array names GMIBox $path*] { unset GMIBox($n) }
+	return
+    }
+    if { [set nargs [llength $args]] != 0 } {
+	foreach "a1 a2 a3 a4 a5" $args { break }
+    }
+    if { $act != "create" } {
+	if { [catch {set csize $GMIBox($path,csize)}] } {
+	    BUG trying to use non-existing ImageListbox	
+	}
+	set end $csize
+	if { $end > 0 } { incr end -1 }
+	foreach x "sel eh ew mode" {
+	    set $x $GMIBox($path,$x)
+	}
+    }
+    set res ""
+    switch $act {
+	create {
+	    # SIZE WIDTH EHEIGHT MODE ?SCROLLBAR?
+	    if { $nargs < 4 } { BUG missing args to ImageListbox create }
+	    if { $a3 < 5 } { set a3 5 }
+	    set height [expr $a1*$a3]
+	    foreach x "csize sel eh ew mode anchor base height" \
+		    v "0 {} $a3 $a2 $a4 {} 0 $height" {
+		set GMIBox($path,$x) $v
+	    }
+	    canvas $path -height $height -width $a2 -confine 1 \
+		-borderwidth 2 -relief sunken
+	    if { $a5 != "" } {
+		$path configure -yscrollincrement $a3 \
+		    -yscrollcommand "ImageListboxScroll $path $a5" \
+		    -scrollregion "0 0 $a2 $height"
+		trace variable GMIBox($path,csize) w ImageListboxResize
+	    }
+	    $path bind entry <Shift-Button-1> \
+		"ImageListboxESButton $path %y ; break"
+	    $path bind entry <Control-Button-1> \
+		"ImageListboxECButton $path %y ; break"
+	    $path bind entry <Button-1> "ImageListboxEButton $path %y"
+	}
+	insert {
+	    # INDEX IMAGE TEXT ?TAGS?
+	    if { $nargs < 3 } { BUG missing args to ImageListbox insert }
+	    set ih [expr $eh-2]
+	    if { $a2 != "" && \
+		     ([image width $a2] > $ih || [image height $a2] > $ih) } {
+		if { [image type $a2] != "photo" } {
+		    if { $a3 == "" } { return -1 }
+		    set a2 ""
+		} else {
+		    if { [catch {set im $GMIBox(img,for,$a2)}] } {
+			set im [image create photo -width $ih -height $ih]
+			$im copy $a2 -from 0 0 $ih $ih
+			set GMIBox(img,for,$a2) $im
+		    }
+		    set a2 $im
+		}
+	    }
+	    if { $a1 != "end" } {
+		set na1 [ImageListboxIndices $path $end $a1]
+		if { $a1 > $na1 } {
+		    # assuming given index must be an integer
+		    set na1 $csize
+		}
+		# update selection
+		set s ""
+		foreach e $GMIBox($path,sel) {
+		    if { $e >= $na1 } { incr e }
+		    lappend s $e
+		}
+		set GMIBox($path,sel) $s
+		# move lower entries down
+		set y0 [expr $na1*$eh]
+		if { $csize > 0  && $na1 < $csize } {
+		    foreach it [$path find withtag entry] {
+			if { [lindex [$path coords $it] 1] >= $y0 } {
+			    $path move $it 0 $eh
+			}
+		    }
+		}
+	    } else { set y0 [expr $csize*$eh] }
+	    $path create rectangle 1 [expr $y0+1] $ew [expr $y0+$eh] \
+		-fill $COLOUR(bg) -outline $COLOUR(bg) \
+		-tags [list txt entry bg "tgs=$a4"]
+	    # texts are created even if empty so that they can be retrieved
+	    $path create text [expr $eh+8] [expr $y0+$eh/2] -anchor w \
+		-text $a3 -fill $COLOUR(fg) \
+		-tags [list txt entry "txt=$a3"]
+	    if { $a2 != "" } {
+		$path create image 5 [expr $y0+2] -anchor nw -image $a2 \
+		    -tags "img entry"
+	    }
+	    incr GMIBox($path,csize)
+	    update idletasks
+	}
+	delete {
+	    # INDEX ?INDEX?
+	    if { $nargs < 1 } { BUG missing args to ImageListbox delete }
+	    foreach "a1 a2" [ImageListboxIndices $path $end [list $a1 $a2]] {}
+	    if { $a2 == "" } { set a2 $a1 }
+	    if { [set ndel [expr $a2-$a1+1]] == $csize } {
+		$path delete all
+		set GMIBox($path,sel) ""
+	    } else {
+		# update selection
+		set s ""
+		foreach e $GMIBox($path,sel) {
+		    if { $e < $a1 } {
+			lappend s $e
+		    } elseif { $e > $a2 } {
+			lappend s [expr $e-$ndel]
+		    }
+		}
+		set GMIBox($path,sel) $s
+		# move lower entries up
+		set y0 [expr $a1*$eh] ; set yn [expr ($a2+1)*$eh]
+		set dy [expr -$ndel*$eh]
+		foreach it [$path find withtag entry] {
+		    if { [set y [lindex [$path coords $it] 1]] >= $y0 } {
+			if { $y >= $yn } {
+			    $path move $it 0 $dy
+			} else { $path delete $it }
+		    }
+		}
+	    }
+	    set GMIBox($path,csize) [expr $csize-$ndel]
+	    update idletasks
+	}
+	get {
+	    # INDEX ?INDEX?
+	    if { $nargs < 1 } { BUG missing args to ImageListbox get }
+	    foreach "a1 a2" [ImageListboxIndices $path $end [list $a1 $a2]] {}
+	    if { $a2 == "" } { set a2 $a1 }
+	    set y0 [expr $a1*$eh] ; set yn [expr ($a2+1)*$eh]
+	    set r ""
+	    foreach it [$path find withtag txt] {
+		if { [set y [lindex [$path coords $it] 1]] >= $y0 && \
+			 $y < $yn } {
+		    foreach t [$path gettags $it] {
+			if { [regsub {^txt=} $t "" tx] } {
+			    lappend r [list [expr round($y)] $tx]
+			    break
+			}
+		    }
+		}
+	    }
+	    foreach p [lsort -integer -index 0 $r] {
+		lappend res [lindex $p 1]
+	    }
+	}
+	gettags {
+	    # INDEX ?INDEX?
+	    if { $nargs < 1 } { BUG missing args to ImageListbox gettags }
+	    foreach "a1 a2" [ImageListboxIndices $path $end [list $a1 $a2]] {}
+	    if { $a2 == "" } { set a2 $a1 }
+	    set y0 [expr $a1*$eh] ; set yn [expr ($a2+1)*$eh]
+	    set r ""
+	    foreach it [$path find withtag txt] {
+		if { [set y [lindex [$path coords $it] 1]] >= $y0 && \
+			 $y < $yn } {
+		    foreach t [$path gettags $it] {
+			if { [regsub {^tgs=} $t "" tgs] } {
+			    lappend r [list [expr round($y)] $tgs]
+			    break
+			}
+		    }
+		}
+	    }
+	    foreach p [lsort -integer -index 0 $r] {
+		lappend res [lindex $p 1]
+	    }
+	}
+	selset {
+	    # INDEX ?INDEX?
+	    # add to selection, irrespective of $mode
+	    # keep selection list ordered
+	    if { $nargs < 1 } { BUG missing args to ImageListbox selset }
+	    foreach "a1 a2" [ImageListboxIndices $path $end [list $a1 $a2]] {}
+	    if { $a2 == "" } { set a2 $a1 }
+	    set y0 [expr $a1*$eh+2]
+	    set s ""
+	    foreach ix $sel {
+		if { $ix == $a1 } {
+		    if { [incr a1] > $a2 } {
+			set a1 1e10
+		    } else { set y0 [expr $y0+$eh] }
+		} else {
+		    while { $a1 < $ix } {
+			ImageListboxSelect sel $path $y0
+			lappend s $a1
+			if { [incr a1] > $a2 } {
+			    set a1 1e10
+			} else { set y0 [expr $y0+$eh] }
+		    }
+		}
+		lappend s $ix
+	    }
+	    while { $a1 <= $a2 } {
+		ImageListboxSelect sel $path $y0
+		lappend s $a1
+		incr a1 ; set y0 [expr $y0+$eh]
+	    }
+	    set GMIBox($path,sel) $s
+	    update idletasks
+	}
+	selclr {
+	    # INDEX ?INDEX?
+	    # keep selection list ordered
+	    if { $nargs < 1 } { BUG missing args to ImageListbox selclr }
+	    foreach "a1 a2" [ImageListboxIndices $path $end [list $a1 $a2]] {}
+	    if { $a2 == "" } { set a2 $a1 }
+	    set s ""
+	    foreach ix $sel {
+		if { $ix >= $a1 && $ix <= $a2 } {
+		    ImageListboxSelect clear $path [expr $ix*$eh+2]
+		} else { lappend s $ix }
+	    }
+	    set GMIBox($path,sel) $s
+	    update idletasks
+	}
+	cursel {
+	    set res $sel
+	}
+	getsel {
+	    set dy [expr $eh-1]
+	    foreach ix $sel {
+		set y0 [expr $ix*$eh]
+		foreach it [$path find overlapping 0 $y0 100 [expr $y0+$dy]] {
+		    foreach t [$path gettags $it] {
+			if { [regsub {^txt=} $t "" tx] } {
+			    lappend res $tx
+			    break
+			}
+		    }
+		}
+	    }
+	}
+	getseltags {
+	    foreach ix $sel {
+		set y0 [expr ($ix+0.5)*$eh]
+		foreach it [$path find overlapping 0 $y0 100 [expr $y0+4]] {
+		    foreach t [$path gettags $it] {
+			if { [regsub {^tgs=} $t "" tgs] } {
+			    lappend res $tgs
+			    break
+			}
+		    }
+		}
+	    }
+	}
+	seldel {
+	    set dy [expr -$eh]
+	    foreach ix [lsort -integer -decreasing $sel] {
+		# move lower entries up
+		set y0 [expr $ix*$eh] ; set yn [expr $y0+$eh]
+		foreach it [$path find withtag entry] {
+		    if { [set y [lindex [$path coords $it] 1]] >= $yn } {
+			$path move $it 0 $dy
+		    } elseif { $y >= $y0 } { $path delete $it }
+		}
+	    }
+	    set GMIBox($path,csize) [expr $csize-[llength $sel]]
+	    set GMIBox($path,sel) ""
+	    update idletasks
+	}
+	index {
+	    # Y (coordinates inside listbox)
+	    if { $nargs < 1 } { BUG missing args to ImageListbox index }
+	    set res [expr int($a1/$GMIBox($path,eh))+$GMIBox($path,base)]
+	    if { $res > $end } { set res $end }
+	}
+	default { BUG calling ImageListbox with wrong action }
+    }
+    return $res
+}
+
+proc ImageListboxScroll {path scr pos0 posf} {
+    # scrolling image listbox
+    #  $scr is scrollbar
+    #  $pos0, $posf are the arguments to the scrolling command
+    #     percentage of vertical dimension for top and bottom positions
+    global GMIBox
+
+    set s $GMIBox($path,csize)
+    set GMIBox($path,base) [expr round($s*$pos0)]
+    $scr set $pos0 $posf
+    return
+}
+
+proc ImageListboxResize {a aix op} {
+    # called by trace when $GMIBox($path,csize) has been changed
+    # resize scroll region after a change of size in image listbox
+    global GMIBox
+
+    regsub {,csize} $aix "" path
+    if { ! [winfo exists $path] } { return }
+    if { [set nh [expr $GMIBox($path,csize)*$GMIBox($path,eh)]] < \
+	     [set max $GMIBox($path,height)] } {
+	set nh $max
+    }
+    $path configure -scrollregion "0 0 $GMIBox($path,ew) $nh"
+    return
+}
+
+proc ImageListboxSelect {act path y0} {
+    # change aspect of entry when selection state changes
+    #  $act in {sel, clear}
+    global GMIBox COLOUR
+
+    if { $act != "sel" } { set act "" }
+    foreach it [$path find overlapping 0 $y0 100 [expr $y0+4]]] {
+	if { [lsearch -exact [$path gettags $it] bg] != -1 } {
+	    $path itemconfigure $it -fill $COLOUR(${act}bg)
+	    break
+	}
+    }
+    return
+}
+
+proc ImageListboxIndices {path end ixs} {
+    # check indices of image-listbox widget that must be in (non-strict)
+    #  increasing order
+    #  $ixs is list of non-negative integers, "end", or "" (discarded)
+    #  $end is either 0 or current size-1
+    # return list of integers in the 0..$end range
+
+    set r "" ; set min 0
+    foreach ix $ixs {
+	if { $ix == "" } { continue }
+	if { $ix == "end" } {
+	    set ix $end
+	} elseif { ! [regexp {^[1-9]*[0-9]+$} $ix] } {
+	    if { $ix != "end" } { BUG bad index for ImageListbox }
+	    set ix $end
+	} elseif { $ix > $end } {
+	    set ix $end
+	}
+	if { $ix < $min } { BUG bad index for ImageListbox }
+	set min $ix
+	lappend r $ix
+    }
+    return $r
+}
+
+proc ImageListboxEButton {path y} {
+    # mouse button-1 on ImageListbox entry
+    global GMIBox
+
+    if { $GMIBox($path,sel) != "" } { ImageListbox selclr $path 0 end }
+    set ix [expr int($y/$GMIBox($path,eh))+$GMIBox($path,base)]
+    ImageListbox selset $path $ix
+    set GMIBox($path,anchor) $ix
+    return
+}
+
+proc ImageListboxECButton {path y} {
+    # mouse control-button-1 on ImageListbox entry
+    global GMIBox
+
+    if { $GMIBox($path,mode) != "extended" } { return }
+    set ix [expr int($y/$GMIBox($path,eh))+$GMIBox($path,base)]
+    if { [lsearch -exact $GMIBox($path,sel) $ix] != -1 } {
+	ImageListbox selclr $path $ix
+	set GMIBox($path,anchor) ""
+    } else {
+	ImageListbox selset $path $ix
+	set GMIBox($path,anchor) $ix
+    }
+    return
+}
+
+proc ImageListboxESButton {path y} {
+    # mouse shift-button-1 on ImageListbox entry
+    global GMIBox
+
+    if { $GMIBox($path,mode) != "extended" } { return }
+    if { [set a $GMIBox($path,anchor)] == "" } {
+	ImageListboxEButton $path $y
+	return
+    }
+    if { [set ix [expr int($y/$GMIBox($path,eh))+$GMIBox($path,base)]] \
+	     != $a } {
+	if { $ix > $a } {
+	    ImageListbox selset $path $a $ix
+	} else { ImageListbox selset $path $ix $a }
+    }
+    set GMIBox($path,anchor) $ix
+    return
+}
 
