@@ -17,7 +17,7 @@
 #      along with this program.
 #
 #  File: know.tcl
-#  Last change:  25 March 2004
+#  Last change:  22 October 2004
 #
 # Includes contributions by Brian Baulch (baulchb@onthenet.com.au)
 #  marked "BSB contribution"
@@ -52,14 +52,49 @@ proc CreateItem {wh data} {
     set ix $Index($wh)
     incr Index($wh) ; incr Number($wh) ; incr Number(Data)
     SetItem $wh $ix $data ; ListAdd $wh $ix
-    if { $Number(Data) == 1 } { ChangeOnStateData normal }
-    if { "$wh" == "WP" } { set WPRoute($ix) "" }
+    if { $Number(Data) == 1 } { ChangeOnState datastate normal }
+    if { $wh == "WP" } { set WPRoute($ix) "" }
     return $ix
 }
 
+proc CreateGRFor {iname obs lp} {
+    # create a GR
+    #  $iname is index of TXT to use as prefix for GR name, or is a literal
+    #    prefix if given as =PREFIX, or has the form @NAME for a
+    #    literal name (if in use, the existing GR will be replaced)
+    #  $obs is GR remark
+    #  $lp is GR contents
+    # return index of new GR
+    global TXT GRDispl
+
+    if { [regsub {^@} $iname "" grname] } {
+	set ix [IndexNamed GR $grname]
+	set data [FormData GR "Name Obs Conts" [list $grname $obs $lp]]
+	if { $ix != -1 } {
+	    if { $GRDispl($ix) } { UnMapGR $ix }
+	    SetItem GR $ix $data
+	    UpdateItemWindows GR $ix
+	} else { set ix [CreateItem GR $data] }
+	return $ix
+    }
+    if { ! [regsub {^=} $iname "" pre] } { set pre $TXT($iname) }
+    set n 0
+    while 1 {
+	set grname [format "$pre %d" $n]
+	if { [IndexNamed GR $grname] == -1 } {
+	    set data [FormData GR "Name Obs Conts" [list $grname $obs $lp]]
+	    return [CreateItem GR $data]
+	    break
+	}
+	incr n
+    }
+    # not used
+    return
+}
+
 proc ItemData {wh index} {
     # find data for item with given index
-    #  $wh in $TYPES
+    #  $wh in $TYPES or LAP
     # return list of values in the order given by $Storage($wh)
     # see GMStart (main.tcl) for the description of data arrays
     global Storage
@@ -68,17 +103,17 @@ proc ItemData {wh index} {
     foreach s $Storage($wh) {
 	global $s
 
-	set l [lappend l "[set [set s]($index)]"]
+	set l [lappend l [set [set s]($index)]]
     }
     return $l
 }
 
 proc FormData {wh names vals} {
-    # create a data list for an item of type $wh (in $TYPES, or TP or LP)
+    # create a data list for an item of type $wh (in $TYPES, or LAP, TP, LP)
     #  $names is a list of data array names without the prefix $wh
     #  $vals is a list of values aligned with $names
     # return list of values in the order given by $DataIndex($wh)
-    #  (if $wh in $TYPES that is the order of $Storage($wh)) using
+    #  (if $wh in $TYPES or LAP that is the order of $Storage($wh)) using
     #  default values for those not given in $vals
     # see GMStart (main.tcl) for the description of data arrays
     global DataDefault DataIndex
@@ -93,7 +128,7 @@ proc FormData {wh names vals} {
 
 proc SetItem {wh index data} {
     # set data for item with given index
-    #  $wh in $TYPES
+    #  $wh in $TYPES or LAP
     # see GMStart (main.tcl) for description of data arrays
     global Storage IndexOf
 
@@ -102,7 +137,7 @@ proc SetItem {wh index data} {
 
     set name [lindex $data 0]
     if { ! [catch {set oldname [set [set ids]($index)]}] && \
-	    "$oldname" != "$name" } {
+	    $oldname != $name } {
 	unset IndexOf($wh,$oldname)
     }
     set IndexOf($wh,$name) $index
@@ -116,7 +151,7 @@ proc SetItem {wh index data} {
 
 proc UnsetItem {wh index} {
     # destroy data for item with given index
-    #  $wh in $TYPES
+    #  $wh in $TYPES or LAP
     # see GMStart (main.tcl) for description of data arrays
     global Storage IndexOf
 
@@ -134,7 +169,7 @@ proc UnsetItem {wh index} {
 
 proc UnsetSeveral {wh ixs} {
     # destroy data for items with given indices
-    #  $wh in $TYPES
+    #  $wh in $TYPES or LAP
     # see GMStart (main.tcl) for description of data arrays
     global Storage IndexOf
 
@@ -156,7 +191,7 @@ proc UnsetSeveral {wh ixs} {
 
 proc UnsetAll {wh} {
     # destroy data for all items with given type
-    #  $wh in $TYPES
+    #  $wh in $TYPES or LAP
     # see GMStart (main.tcl) for description of data arrays
     global Storage IndexOf
 
@@ -172,19 +207,20 @@ proc UnsetAll {wh} {
 }
 
 proc Forget {wh ix} {
-    # forget an item with given index; $wh in $TYPES
+    # forget an item with given index; $wh in $TYPES or LAP
     global ${wh}Displ RTIdNumber RTWPoints Number MESS TXT
     # BSB contribution
     global MYGPS WPName WPNum UnusedICInx UnusedWPInx
 
-    if { [set ${wh}Displ($ix)] && ![UnMap $wh $ix] && "$wh" != "GR" } {
+    if { [set ${wh}Displ($ix)] && $wh != "LAP" && ! [UnMap $wh $ix] && \
+	     $wh != "GR" } {
 	GMMessage [format $MESS(cantfgt) $TXT(name$wh)]
 	return 0
     }
     switch $wh {
 	WP {
 	    # BSB contribution
-	    if { "$MYGPS" == "Lowrance" } {
+	    if { $MYGPS == "Lowrance" } {
 		if { [string match "ICON*" $WPName($ix)] } {
 		    lappend UnusedICInx $WPNum($ix)
 		} else {
@@ -198,45 +234,49 @@ proc Forget {wh ix} {
     }
     ListDelete $wh $ix ; UnsetItem $wh $ix
     incr Number($wh) -1 ; incr Number(Data) -1
-    if { $Number(Data) == 0 } { ChangeOnStateData disabled }
+    if { $Number(Data) == 0 } { ChangeOnState datastate disabled }
     return 1
 }
 
 proc ForgetSeveral {wh ixs} {
-    # forget several items with given indices; $wh in $TYPES
+    # forget several items with given indices; $wh in $TYPES or LAP
     #  $ixs has the same order of $ListInds($wh) although with some
     #    elements missing
     # proc based on proc Forget
     global ${wh}Displ RTIdNumber RTWPoints Number MESS TXT \
 	    MYGPS WPName WPNum UnusedICInx UnusedWPInx
 
-    set fs "" ; set cf 0 ; set nf 0
-    foreach ix $ixs {
-	if { [set ${wh}Displ($ix)] && ![UnMap $wh $ix] && "$wh" != "GR" } {
-	    set nf 1
-	    continue
-	}
-	switch $wh {
-	    WP {
-		if { "$MYGPS" == "Lowrance" } {
-		    if { [string match "ICON*" $WPName($ix)] } {
-			lappend UnusedICInx $WPNum($ix)
-		    } else {
-			lappend UnusedWPInx $WPNum($ix)
+    if { $wh == "GR" || $wh == "LAP" } {
+	set fs $ixs ; set cf [expr -[llength $ixs]]
+    } else {
+	set fs "" ; set cf 0; set nf 0
+	foreach ix $ixs {
+	    if { [set ${wh}Displ($ix)] && ! [UnMap $wh $ix] } {
+		set nf 1
+		continue
+	    }
+	    switch $wh {
+		WP {
+		    if { $MYGPS == "Lowrance" } {
+			if { [string match "ICON*" $WPName($ix)] } {
+			    lappend UnusedICInx $WPNum($ix)
+			} else {
+			    lappend UnusedWPInx $WPNum($ix)
+			}
 		    }
 		}
+		RT {
+		    UnsetWPRoute $RTIdNumber($ix) $RTWPoints($ix)
+		}
 	    }
-	    RT {
-		UnsetWPRoute $RTIdNumber($ix) $RTWPoints($ix)
-	    }
+	    lappend fs $ix
+	    incr cf -1
 	}
-	lappend fs $ix
-	incr cf -1
+	if { $nf } { GMMessage [format $MESS(cantfgt) $TXT(name$wh)] }
     }
-    if { $nf } { GMMessage [format $MESS(cantfgt) $TXT(name$wh)] }
     incr Number($wh) $cf ; incr Number(Data) $cf
     if { $Number($wh) == 0 } {
-	if { $Number(Data) == 0 } { ChangeOnStateData disabled }
+	if { $Number(Data) == 0 } { ChangeOnState datastate disabled }
 	ChangeOnStateList $wh disabled
 	UnsetAll $wh ; ListDeleteAll $wh
     } else {
@@ -272,7 +312,7 @@ proc AllIndicesForType {wh types} {
 }
 
 proc IndexNamed {wh name} {
-    # find index for item with given name; $wh in $TYPES
+    # find index for item with given name; $wh in $TYPES or LAP
     global IndexOf
 
     if { [catch {set ix $IndexOf($wh,$name)}] } {
@@ -282,7 +322,7 @@ proc IndexNamed {wh name} {
 }
 
 proc NameOf {wh ix} {
-    # return name of item with given index; $wh in $TYPES
+    # return name of item with given index; $wh in $TYPES or LAP
     global Storage
 
     set ids [lindex $Storage($wh) 0]
@@ -341,7 +381,7 @@ proc SetWPRoute {rt wps} {
     #  whose name belongs to $wps
     global WPRoute
 
-    foreach wp "$wps" {
+    foreach wp $wps {
 	if { [set ix [IndexNamed WP $wp]] != -1 } {
 	    if { [lsearch -exact $WPRoute($ix) $rt] == -1 } {
 		set WPRoute($ix) [linsert $WPRoute($ix) 0 $rt]
@@ -357,7 +397,7 @@ proc UnsetWPRoute {rt wps} {
     #  whose name belongs to $wps
     global WPRoute
 
-    foreach wp "$wps" {
+    foreach wp $wps {
 	set ix [IndexNamed WP $wp]
 	if { $ix != -1 } {
 	    set wi [lsearch -exact $WPRoute($ix) $rt]
@@ -373,20 +413,20 @@ proc DateCommt {date} {
     # create comment from date
     global COMMENTLENGTH NOLOWERCASE
 
-    regsub -all {:|\.} "$date" "" date
-    if { [string length "$date"] > $COMMENTLENGTH } {
+    regsub -all {:|\.} $date "" date
+    if { [string length $date] > $COMMENTLENGTH } {
 	set date [string range "$date" 0 [expr $COMMENTLENGTH-1]]
     }
     if { $NOLOWERCASE } {
 	return [string toupper "$date"]
     }
-    return "$date"
+    return $date
 }
 
 ## operations on groups
 
 proc GRsElements {ixs rec wh} {
-    # find elements of type $wh (in $TYPES) in groups with
+    # find elements of type $wh (in $TYPES or LAP) in groups with
     #  given indices; if $wh==GR the initial GRs are included in the result;
     #  undefined elements are not included
     #  $rec is 1 if search is recursive
@@ -394,7 +434,7 @@ proc GRsElements {ixs rec wh} {
     global GMember
 
     catch { unset GMember }
-    if { "$wh" == "GR" } {
+    if { $wh == "GR" } {
 	foreach ix $ixs { set GMember($ix) 1 }
     }
     GRsElsCollect $ixs $rec $wh
@@ -404,7 +444,7 @@ proc GRsElements {ixs rec wh} {
 }
 
 proc GRsElsCollect {ixs rec wh} {
-    # mark defined elements of type $wh (in $TYPES) in groups with
+    # mark defined elements of type $wh (in $TYPES or LAP) in groups with
     #  given indices
     #  $rec is 1 if search is recursive
     # marked elements with index $i will have GMember($i) set
@@ -412,7 +452,7 @@ proc GRsElsCollect {ixs rec wh} {
 
     foreach ix $ixs {
 	foreach p $GRConts($ix) {
-	    if { "[lindex $p 0]" == "$wh" } {
+	    if { [lindex $p 0] == $wh } {
 		foreach e [lindex $p 1] {
 		    if { [set eix [IndexNamed $wh $e]] != -1 } {
 			set GMember($eix) 1
@@ -420,7 +460,7 @@ proc GRsElsCollect {ixs rec wh} {
 		}
 		if { ! $rec } { break }
 	    }
-	    if { $rec && "[lindex $p 0]" == "GR" } {
+	    if { $rec && [lindex $p 0] == "GR" } {
 		set rixs [Apply [lindex $p 1] IndexNamed GR]
 		while { [set i [lsearch -exact $rixs -1]] != -1 } {
 		    set rixs [lreplace $rixs $i $i]
@@ -450,7 +490,7 @@ proc EndWPRenaming {} {
     # this proc must be called after any input operation that stored data!
     # build a group with renamed WPs and RTs or GRs that have ambiguous
     #  WPs, after an input operation (get, load, import)
-    global ReplacedNames ReplacedWPs ReplRTs ReplGRs TXT
+    global ReplacedNames ReplacedWPs ReplRTs ReplGRs
 
     if { $ReplacedNames == "" && $ReplacedWPs == "" } { return }
     set nwps ""
@@ -470,11 +510,7 @@ proc EndWPRenaming {} {
 	    lappend lp [list $wh [lsort -dictionary $ns]]
 	}
     }
-    set grname [format "$TXT(renres) %d" 0] ; set n 0
-    while { [IndexNamed GR $grname] != -1 } {
-	set grname [format "$TXT(renres) %d" [incr n]]
-    }
-    CreateItem GR [FormData GR "Name Conts" [list $grname $lp]]
+    CreateGRFor renres "" $lp
     return
 }
 
@@ -596,8 +632,7 @@ proc SamePosnDat {posndat1 posndat2} {
     foreach "lat1 long1 dat1" $posndat1 { break }
     foreach "lat2 long2 dat2" $posndat2 { break }
     if { $posndat1 != $posndat2 } {
-	set p [ConvertDatum $lat2 $long2 $dat2 $dat1 DDD]
-	set lat2 [lindex $p 0] ; set long2 [lindex $p 0]
+	foreach "lat2 long2" [ToDatum $lat2 $long2 $dat2 $dat1] { break }
     }
     return [expr $lat1 == $lat2 && $long1 == $long2]
 }
@@ -635,8 +670,8 @@ proc SamePosn {ix data} {
     set ip $DataIndex(WPPosn)
     set id $DataIndex(WPDatum)
     set p [lindex $data $ip] ; set d [lindex $data $id]
-    if { "$WPDatum($ix)" != "$d" } {
-	set p [ConvertDatum [lindex $p 0] [lindex $p 1] $d $WPDatum($ix) DDD]
+    if { $WPDatum($ix) != $d } {
+	set p [ToDatum [lindex $p 0] [lindex $p 1] $d $WPDatum($ix)]
     }
     if { [lindex $p 0] == [lindex $WPPosn($ix) 0] && \
 	    [lindex $p 1] == [lindex $WPPosn($ix) 1] } { return 1 }
@@ -753,6 +788,16 @@ proc StoreLN {ix id data todispl} {
     return
 }
 
+proc StoreLAP {ix name data} {
+    # store LAP data just read in assumed to be a new lap
+
+    if { $ix != -1 } {
+	SetItem LAP $ix $data
+	UpdateItemWindows LAP $ix
+    } else { set ix [CreateItem LAP $data] }
+    return $ix
+}
+
 proc StoreGR {ix id data todispl} {
     # store GR data just read in
     #  $todispl is true if the GR should be mapped
