@@ -17,11 +17,11 @@
 #      along with this program.
 #
 #  File: langengl.tcl
-#  Last change:  1 June 2004
+#  Last change:  16 October 2004
 #
 # French language file by Valre Robin (valere.robin@wanadoo.fr)
 #  Last change by him:  11 November 2001
-#  last input from him inserted:  1 June 2004
+#  last input from him inserted:  13 July 2004
 #
 # Includes contributions by Jean H Theoret (ve2za@rac.ca)
 # Includes typo corrections by Laurent Bonnaud (bonnaud@lis.inpg.fr)
@@ -259,7 +259,8 @@ array set MESS {
     unobscmap "Erreur probablement cause par le recouvrement de la carte par une icne ou une fentre: recommencer aprs un dlai."
     cantwrtimg "Erreur d'criture dans un fichier au format %s"
     TRsimplres "Cration d'une trace simplifie %s"
-    cantsaveRTid "Route %s non sauvegarde : identifiant non numrique"
+    cantsaveRTid "%s route(s) non sauvegarde(s) : identifiant non numrique"
+    cantsaveTRid "%s trace(s) non sauvegard(s) : identifiant non numrique"
     badtrvconf "Configuration corrompue; recommencer avec une configuration vide"
     drvsimoff "Simulateur de conduite : pas encore dmarr"
     needWP "Simulateur de conduite : charger ou dfinir des points de route"
@@ -1030,7 +1031,6 @@ array set TXT {
     Ghidden_map_area "Zone de carte"
     Ghidden_map_int "Intersection de carte"
     Ghidden_map_addr "Adresse de carte"
-    Ghidden_map_label "tiquette de carte"
     Ghidden_map_line "Ligne de carte"
     Ghidden_locked "Verrouill"
     Ghidden_default "Dfaut"
@@ -1089,6 +1089,10 @@ array set TXT {
     ozimapfile "Ficher Ozi Map"
     info "Information"
     climbrate "Vitesse verticale"
+    optUSESLOWOPWINDOW "Fentre de contrle des oprations lentes"
+    optDEFTRECPROTOCOL "Protocole par dfaut"
+    optLNSREACT "Les lignes sur la carte sont cliquable  la souris"
+    syhlpbx "Utiliser le bouton droit pour\nrordonnancer les lments de la liste"
 }
 
     # the following definitions must be coherent with $TXT
