@@ -17,9 +17,9 @@
 #      along with this program.
 #
 #  File: langnl.tcl
-#  Last change:  1 June 2004
+#  Last change:  22 October 2004
 #  Dutch language file by Rob Buitenhuis (rob@buitenhs.demon.nl)
-#  last input from him inserted: 1 June 2004
+#  last input from him inserted: 22 October 2004
 #
 # 
 #  file langengl.tcl is consulted first; no need for duplicating entries here
@@ -268,6 +268,7 @@ array set MESS {
     cantwrtimg "Fout tijdens schrijven van beeld naar bestand in %s formaat"
     TRsimplres "TR als resultaat van vereenvoudiging gemaakt met naam %s"
     cantsaveRTid "%d RT(s) niet opgeslagen: niet numerieke aanduiding"
+    cantsaveTRid "%d TR(s) niet opgeslagen: niet numerieke aanduiding"
     badtrvconf "Corrupte configuratie; herstart met een lege"
     drvsimoff "Rij-simulator: nog niet gestart!"
     needWP "Rij-simulator: laad of definieer aub eerst wat WP's"
@@ -280,6 +281,12 @@ array set MESS {
     RTnoWPname "WP in RT met alleen een naam wordt niet langer ondersteund"
     distlarge "Afstand te groot!"
     timeconsmg "Bewerking kost tijd: doorgaan?"
+    badtimeval "Fout in waarde tijd"
+    badLAP "Fout in ronde; regel"
+    lapncnsdrd "Rondes niet meenemen"
+    undefinedTR   "Onbekende TR"
+    emptymenu "Menu leeg; doorgaan?"
+    cantwrtsymenu "Kan menu met symbolen niet wegschrijven"
 }
 
 set TXT(RTcompflds) "# WP {$DTUNIT} graad {$ALUNIT} et. beschr."
@@ -979,62 +986,65 @@ array set TXT {
     rthlpdsim "Pijltjes-toetsen: sturen, verander snelheid\nSpatiebalk: rechtuit"
     renres "HERNOEMD"
     chggoal "Volgend doel instellen"
-    Ghidden_class Klasse
-    Ghidden_subclass Subklasse
-    Ghidden_lnk_ident "Traject id"
-    Ghidden_colour Kleur
-    Ghidden_attrs Attributen
-    Ghidden_depth Diepte
-    Ghidden_state Staat
-    Ghidden_country Land
-    Ghidden_facility Faciliteit
-    Ghidden_city Stad
     Ghidden_addr Addres
-    Ghidden_int_road "Kruisende weg"
-    Ghidden_dtyp "Display opt+type"
-    Ghidden_ete "ETE"
-    Ghidden_display "Display?"
-    Ghidden_yes Ja
-    Ghidden_no Nee
-    Ghidden_user "Gebruiker"
-    Ghidden_user_symbol "Gebruiker (alleen symbool)"
-    Ghidden_non_user "Niet-gebruiker"
-    Ghidden_avn_airport "Vliegveld"
-    Ghidden_avn_inters "Kruispunt luchtwegen"
+    Ghidden_attrs Attributen
     Ghidden_avn_NDB "NDB"
     Ghidden_avn_VOR "VOR"
-    Ghidden_avn_airp_rway "Drempel landingsbaan"
-    Ghidden_avn_airp_int "Kruispunt vliegveld"
     Ghidden_avn_airp_NDB "Vliegveld NDB"
-    Ghidden_map_pt "Kaart punt"
-    Ghidden_map_area "Kaart gebied"
-    Ghidden_map_int "Kaart kruispunt"
-    Ghidden_map_addr "Kaart addres"
-    Ghidden_map_label "Kaart label"
-    Ghidden_map_line "Kaart lijn"
-    Ghidden_locked Vastgezet
-    Ghidden_default "Verstek"
+    Ghidden_avn_airp_int "Kruispunt vliegveld"
+    Ghidden_avn_airp_rway "Drempel landingsbaan"
+    Ghidden_avn_airport "Vliegveld"
+    Ghidden_avn_inters "Kruispunt luchtwegen"
     Ghidden_black Zwart
-    Ghidden_white Wit
-    Ghidden_red Rood
-    Ghidden_dark_red "Donkerrood"
-    Ghidden_green Groen
-    Ghidden_dark_green "Donkergroen"
     Ghidden_blue Blauw
-    Ghidden_dark_blue "Donkerblauw"
-    Ghidden_yellow Geel
-    Ghidden_dark_yellow "Donkergeel"
-    Ghidden_magenta Magenta
-    Ghidden_dark_magenta "Donkermagenta"
+    Ghidden_cat Kategorie
+    Ghidden_city Stad
+    Ghidden_class Klasse
+    Ghidden_colour Kleur
+    Ghidden_country Land
     Ghidden_cyan Cyaan
+    Ghidden_dark_blue "Donkerblauw"
     Ghidden_dark_cyan "Donkercyaan"
-    Ghidden_light_gray "Lichtgrijs"
     Ghidden_dark_gray "Donkergrijs"
+    Ghidden_dark_green "Donkergroen"
+    Ghidden_dark_magenta "Donkermagenta"
+    Ghidden_dark_red "Donkerrood"
+    Ghidden_dark_yellow "Donkergeel"
+    Ghidden_default "Verstek"
+    Ghidden_depth Diepte
+    Ghidden_direct Direct
+    Ghidden_display "Display?"
+    Ghidden_dtyp "Display opt+type"
+    Ghidden_ete "ETE"
+    Ghidden_facility Faciliteit
+    Ghidden_green Groen
+    Ghidden_int_road "Kruisende weg"
+    Ghidden_light_gray "Lichtgrijs"
     Ghidden_line Lijn
     Ghidden_link Traject
+    Ghidden_lnk_ident "Traject id"
+    Ghidden_locked Vastgezet
+    Ghidden_magenta Magenta
+    Ghidden_map_addr "Kaart addres"
+    Ghidden_map_area "Kaart gebied"
+    Ghidden_map_int "Kaart kruispunt"
+    Ghidden_map_line "Kaart lijn"
+    Ghidden_map_pt "Kaart punt"
     Ghidden_net Rooster
-    Ghidden_direct Direct
+    Ghidden_no Nee
+    Ghidden_non_user "Niet-gebruiker"
+    Ghidden_red Rood
     Ghidden_snap Klik
+    Ghidden_state Staat
+    Ghidden_subclass Subklasse
+    Ghidden_temp Temperatuur
+    Ghidden_time Tijdsmarkering
+    Ghidden_transparent Transparant
+    Ghidden_user "Gebruiker"
+    Ghidden_user_symbol "Gebruiker (alleen symbool)"
+    Ghidden_white Wit
+    Ghidden_yellow Geel
+    Ghidden_yes Ja
     cwpsdef "Alsnog te def. referentie-WP's"
     PRJBMN "Oostenrijks BMN Grid"
     PRJCTR "Carta Tecnica Reg (I)"
@@ -1069,6 +1079,118 @@ array set TXT {
     ozimapfile "Ozi Map-bestand"
     info "Informatie"
     climbrate "Klimsnelheid"
+    BGAfeature  "BGA Feature"
+    BGAfindblty "BGA Findability"
+    BGAairact   "BGA Air activity"
+    optUSESLOWOPWINDOW "Controle-venster langdurige bewerkingen"
+    optDEFTRECPROTOCOL "Standaard-protocol"
+    optLNSREACT "LNs op de kaart met muis bewerken"
+    optDEFMAPPFDATUM "Datum kaartcoordinaten"
+    optAutoNumRts "Automatisch nummeren bij export"
+    mkgrp "Groep maken"
+    dispitems "Weergegeven objecten"
+    hiditems "Afgeschermde objecten"
+    SYopen_24hr "24 u p. dag open"
+    SYfhs_facility "FHS faciliteit"
+    SYbot_cond "bodem condities"
+    SYtide_pred_stn "Station v. voorsp. getij/stroom" 
+    SYanchor_prohib "Verboden te ankeren"
+    SYbeacon "Baken"
+    SYcoast_guard "Kustwacht"
+    SYreef "Rif"
+    SYweedbed "Zeewier"
+    SYdropoff "Kade"
+    SYdock "Dok"
+    SYmarina "Jachthaven"
+    SYbait_tackle "Visbenodigdheden"
+    SYstump "Stronk"
+    SYcntct_ball_cap "Baseball-pet"
+    SYcntct_big_ears "Groot oor"
+    SYcntct_spike "Spijker"
+    SYcntct_goatee "Puntbaard"
+    SYcntct_dreads "Dreadlocks"
+    SYcntct_female1 "Vrouw 1"
+    SYcntct_female2 "Vrouw 2" 
+    SYcntct_female3 "Vrouw 3"
+    SYcntct_ranger "Commando"
+    SYcntct_pirate "Piraat"
+    SYcntct_biker "Fietser"
+    SYcntct_alien "Marsmannetje"
+    SYcntct_bug "Insect"
+    SYcntct_cat "Kat"
+    SYcntct_dog "Hond"
+    SYcntct_pig "Varken"
+    SYhydrant "Brandkraan"
+    SYcircle_blue "Cirkel, blauw"
+    SYcircle_green "Cirkel, groen"
+    SYcircle_red "Cirkel, rood"
+    SYdiamond_blue "Ruit, blauw"
+    SYoval_blue "Ellips, blauw"
+    SYoval_green "Ellips, groen"
+    SYoval_red "Ellips, rood"
+    SYrect_blue "Rechthoek, blauw"
+    SYrect_green "Rechthoek, groen"
+    SYrect_red "Rechthoek, rood"
+    SYsquare_blue "Vierkant, blauw"
+    SYtriangle_blue "Driehoek, blauw"
+    SYtriangle_green "Driehoek, groen"
+    SYtriangle_red "Driehoek, rood"
+    SYletter_a_blue "A, blauw"
+    SYletter_a_green "A, groen"
+    SYletter_a_red "A, rood"
+    SYletter_b_blue "B, blauw"
+    SYletter_b_green "B, groen"
+    SYletter_b_red "B, rood"
+    SYletter_c_blue "C, blauw"
+    SYletter_c_green "C, groen"
+    SYletter_c_red "C, rood"
+    SYletter_d_blue "D, blauw"
+    SYletter_d_green "D, groen"
+    SYletter_d_red "D, rood"
+    SYnumber_0_blue "0, blauw"
+    SYnumber_0_green "0, groen"
+    SYnumber_0_red "0, rood"
+    SYnumber_1_blue "1, blauw"
+    SYnumber_1_green "1, groen"
+    SYnumber_1_red "1, rood"
+    SYnumber_2_blue "2, blauw"
+    SYnumber_2_green "2, groen"
+    SYnumber_2_red "2, rood"
+    SYnumber_3_blue "3, blauw"
+    SYnumber_3_green "3, groen"
+    SYnumber_3_red "3, rood"
+    SYnumber_4_blue "4, blauw"
+    SYnumber_4_green "4, groen"
+    SYnumber_4_red "4, rood"
+    SYnumber_5_blue "5, blauw"
+    SYnumber_5_green "5, groen"
+    SYnumber_5_red "5, rood"
+    SYnumber_6_blue "6, blauw"
+    SYnumber_6_green "6, groen"
+    SYnumber_6_red "6, rood"
+    SYnumber_7_blue "7, blauw"
+    SYnumber_7_green "7, groen"
+    SYnumber_7_red "7, rood"
+    SYnumber_8_blue "8, blauw"
+    SYnumber_8_green "8, groen"
+    SYnumber_8_red "8, rood"
+    SYnumber_9_blue "9, blauw"
+    SYnumber_9_green "9, groen"
+    SYnumber_9_red "9, rood"
+    syhlpbx "Gebuik rechter-button om\nlijst-elementen te herschikken"
+    syusrmenu "Aangepast menu met symbolen"
+    cfgsymenu "Configureer aangepastmenu met symbolen"
+    insmnb   "Voeg sub-menu in voor"
+    insmna   "Voeg sub-menu in achter"
+    opensbmn "Open sub-menu"
+    clssbmn  "Sluit sub-menu"
+    optSUPPORTLAPS "Ondersteuning van rondes"
+    LAP  RD
+    nameLAP "Ronde"
+    duration "Duur"
+    calrs "Calorieen"
+    getLAP  "Ophalen rondes"
+    run "Wedstrijd"
 }
 
     # the following definitions must be coherent with $TXT
@@ -1081,5 +1203,6 @@ array set INVTXT {
     WP   WP
     RT   RT
     TR   TR
+    RD   LAP
     GR   GR
 }
