@@ -1,7 +1,7 @@
 #
 #  gpsman --- GPS Manager: a manager for GPS receiver data
 #
-#  Copyright (c) 2003 Miguel Filgueiras (mig@ncc.up.pt) / Universidade do Porto
+#  Copyright (c) 2004 Miguel Filgueiras (mig@ncc.up.pt) / Universidade do Porto
 #
 #    This program is free software; you can redistribute it and/or modify
 #      it under the terms of the GNU General Public License as published by
@@ -17,7 +17,7 @@
 #      along with this program.
 #
 #  File: magellan.tcl
-#  Last change:  30 October 2003
+#  Last change:  17 September 2004
 #
 # This file contributed by Matt Martin (matt.martin@ieee.org)
 #  Copyright (c) 2003 Matt Martin (matt.martin@ieee.org)
@@ -733,8 +733,10 @@ proc PrepData {} {
 		set p [lindex $TRTPoints($ix) 0]
 		set PkOutSaved [lreplace $TRTPoints($ix) 0 0]
 		if { "$TRDatum($ix)" != "WGS 84" } {
-		    set tmpp [ConvertDatum [lindex $p 0] [lindex $p 1] \
-			       $TRDatum($ix) "WGS 84" DDD]
+		    # MF change: calling FormatPosition instead of ConvertDatum
+		    set tmpp [lindex [FormatPosition [lindex $p 0] \
+					  [lindex $p 1] $TRDatum($ix) \
+					  DDD "WGS 84"] 0]
 		    set p [concat $tmpp [lrange $p 4 end]]
 		    set PkOutSaved [ChangeTPsDatum $PkOutSaved \
 					$TRDatum($ix) "WGS 84"]
@@ -784,8 +786,9 @@ proc PrepMagWPData {ix } {
 
     set p $WPPosn($ix)
     if { "$WPDatum($ix)" != "WGS 84" } {
-	set p [ConvertDatum [lindex $p 0] [lindex $p 1] \
-		   $WPDatum($ix) "WGS 84" DDD]
+	# MF change: calling FormatPosition instead of ConvertDatum
+	set p [lindex [FormatPosition [lindex $p 0] [lindex $p 1] \
+			   $WPDatum($ix) DDD "WGS 84"] 0]
     }
 
     # figure out the symbol code
@@ -1273,12 +1276,14 @@ proc ConvWPData {data} {
 
     set fs {Name Posn Alt Commt Symbol}
     set ps {4 0 2 5 6}
+    # MF change: using FormatPosition instead of CreatePos, needs position
+    #  format in $all and $cnsts
     if { $CREATIONDATE } {
-	set all [linsert $fs 0 Datum Date ]
-	set cnsts [list "WGS 84" [Now]]
+	set all [linsert $fs 0 Datum PFrmt Date]
+	set cnsts [list "WGS 84" "" [Now]]
     } else {
-	set all [linsert $fs 0 Datum ]
-	set cnsts [list "WGS 84"]
+	set all [linsert $fs 0 Datum PFrmt]
+	set cnsts [list "WGS 84" ""]
     }
     foreach d $data {
 	set vs $cnsts
@@ -1286,8 +1291,14 @@ proc ConvWPData {data} {
 	    set v [lindex $d $p]
 	    switch $f {
 		Posn {
-		    set v [CreatePos [lindex $v 0] [lindex $v 1] \
-			    $PositionFormat "WGS 84"]
+		    # MF change: using FormatPosition instead of CreatePos
+		    foreach "latd longd" $v { break }
+		    foreach "v pfmt datum" \
+			[FormatPosition $latd $longd "WGS 84" \
+			     $PositionFormat "" DDD] {
+			    break
+		    }
+		    set all [lreplace $all 0 1 $datum $pfmt]
 		}
 		Symbol {
 		    #set v 1st_aid
@@ -1420,8 +1431,8 @@ proc ConvTPData {data} {
     set fs "" ; set vs ""
 
     #set p [ConvMagPos $data]
-    # MF change: no need for position type in proc CreatePos
-    set p [CreatePos [lindex $data 0] [lindex $data 1] DMS "WGS 84"]
+    # MF change: replacing call to CreatePos by call to FormatLatLong
+    set p [FormatLatLong [lindex $data 0] [lindex $data 1] DMS]
     set dl [ConvMagDate [lindex $data 4] [lindex $data 7]]
     set alt [lindex $data 2]
     lappend fs latd longd latDMS longDMS alt date secs
