@@ -17,16 +17,17 @@
 #      along with this program.
 #
 #  File: main.tcl
-#  Last change:  7 June 2004
+#  Last change:  22 October 2004
 #
 # Includes contributions by
 #  - Brian Baulch (baulchb@onthenet.com.au) marked "BSB contribution"
 #  - Matt Martin (matt.martin@ieee.org) marked "MGM contribution"
 #  - Valere Robin (valere.robin@wanadoo.fr) marked "VR contribution"
+#  - David Gardner (djgardner@users.sourceforge.net) marked "DJG contribution"
 #
 
-set VERSIONDATE(day) 07
-set VERSIONDATE(month) 06
+set VERSIONDATE(day) 22
+set VERSIONDATE(month) 10
 set VERSIONDATE(year) 2004
 
       # these tests here prevent errors in the lang*.tcl files
@@ -136,7 +137,7 @@ if { int($TimeOffset) != $TimeOffset } {
     exit 1
 }
 
-set VERSION 6.1.1
+set VERSION 6.2
 
 set UDatums "" ; set UEllipsoids ""
 set WindowStack ""
@@ -178,7 +179,10 @@ $TXT(contrib):\n\
 *   Alberto Morales (amd77@gulic.org)\n\
 *   Martin Buck (m@rtin-buck.de)\n\
 *   David Kaplan (dmkaplan@ucdavis.edu)\n\
-*   Jean H Theoret (ve2za@rac.ca)\n"
+*   Jean H Theoret (ve2za@rac.ca)\n\
+*   Paul Scorer (p.scorer@leedsmet.ac.uk)\n\
+*   Nigel Orr (gps@river-view.freeserve.co.uk)\n\
+*   David Gardner (djgardner@users.sourceforge.net)"
 
 
     grid config $frc.txt -column 0 -row 1 -sticky nesw
@@ -189,6 +193,12 @@ $TXT(contrib):\n\
 }
 
 ### images needed when starting up
+
+# user-defined symbol menu (high-level description); see symbols.tcl
+set SYUSRMENU ""
+if { [file readable [set UFile(symenu) [file join $USERDIR symenu]]] } {
+    LoadSymbolCustomMenu
+}
 InitSymbolImages
 
 foreach v "Logo SmallLogo" f "gpsman-logo.gif gpsman-logo-small.gif" {
@@ -284,6 +294,7 @@ proc GlobalOptionsBindings {} {
     option add *Menubutton.direction below
     option add *Menu.tearOff 0
     option add *Entry.exportSelection 1
+    option add *Entry.disabledBackground $COLOUR(bg)
     option add *Checkbutton.selectColor $COLOUR(check)
     option add *Radiobutton.selectColor $COLOUR(check)
     option add *Text.wrap word
@@ -308,15 +319,11 @@ proc GMInit {} {
 	    MapWPMoving MapMakingRT MapLoading MapScInitVal MapImageItems \
 	    GetDispl MAPKNOWNPROJS RECPROTOCOLS SYMBOLIMAGE RcMenu \
 	    GPSProtocol GPSProtocolExt RealTimeLogAnim PVTState USvState \
-	    MAPFTSIZES GSHPVersion NoImgLib ImgOutFormats SmallLogo UNIX GMEd
+	    MAPFTSIZES GSHPVersion NoImgLib ImgOutFormats SmallLogo UNIX GMEd \
+	    FILEFORMAT MapScaleButton MapPFDatum AutoNumRts SUPPORTLAPS
 
     GlobalOptionsBindings
 
-    foreach type $TYPES {
-	set EdWindow($type) .gm$type ; set ListInds($type) ""
-	set GMEd($type,Show) 0
-    }
-
     set MapBounds "0 0 $MAPWIDTH $MAPHEIGHT"
     set MapRange(x) $MAPWIDTH ; set MapRange(y) $MAPHEIGHT
     set MapRange(x0) 0 ; set MapRange(y0) 0
@@ -388,15 +395,31 @@ proc GMInit {} {
 	    button .lsw.frmb.frb.load -text $TXT(load) \
 		    -command { LoadFile Data ; \
 		    .lsw.frmb.frb.load configure -state normal }
+	    set m .lsw.frmb.frb.import.m
+	    menubutton .lsw.frmb.frb.import -text $TXT(import) -menu $m
+	    menu $m
+	    foreach fmt $FILEFORMAT(_type,in,data) {
+		$m add command -label $fmt -command "ImportFile Data $fmt"
+	    }
 	    button .lsw.frmb.frb.save -text $TXT(save) -state disabled \
 		    -command { SaveFile all Data ; \
 		    .lsw.frmb.frb.save configure -state normal }
+	    # based on VR contribution
+	    set m .lsw.frmb.frb.export.m
+	    menubutton .lsw.frmb.frb.export -text $TXT(export) -menu $m \
+		-state disabled
+	    menu $m
+	    foreach fmt $FILEFORMAT(_type,out,data) {
+		$m add command -label $fmt -command "ExportFile all $fmt Data"
+	    }
+	    #--
 	    button .lsw.frmb.frb.search -text $TXT(search) -state disabled \
 		    -command { Search ; \
 		    .lsw.frmb.frb.search configure -state normal }
 	    frame .lsw.frmb.frb.sep2 -height 6 -bg $COLOUR(dialbg) -width $t
 	    set WConf(datastate) [list [list \
-		    button [list .lsw.frmb.frb.save .lsw.frmb.frb.search]]]
+		    button [list .lsw.frmb.frb.save .lsw.frmb.frb.export \
+				.lsw.frmb.frb.search]]]
 	    button .lsw.frmb.mpw -text $TXT(nameMap) -width $t \
 		    -command "ToggleWindow .mpw $MPOSX $MPOSY ; \
 		    .lsw.frmb.mpw configure -state normal"
@@ -441,7 +464,7 @@ proc GMInit {} {
 	    set m .lsw.frbl.frb.import.m
 	    menubutton .lsw.frbl.frb.import -text $TXT(import) -menu $m
 	    menu $m
-	    foreach fmt "GTrackMaker GPX" {
+	    foreach fmt $FILEFORMAT(_type,in,data) {
 		$m add command -label $fmt -command "ImportFile Data $fmt"
 	    }
 	    button .lsw.frbl.frb.save -text $TXT(save) -state disabled \
@@ -452,7 +475,7 @@ proc GMInit {} {
 	    menubutton .lsw.frbl.frb.export -text $TXT(export) -menu $m \
 		-state disabled
 	    menu $m
-	    foreach fmt "GPX" {
+	    foreach fmt $FILEFORMAT(_type,out,data) {
 		$m add command -label $fmt -command "ExportFile all $fmt Data"
 	    }
 	    #--
@@ -479,17 +502,20 @@ proc GMInit {} {
 	    -width [expr [$MpW.frm.frmap1.mvscr cget -width]+4]
 
     frame $MpW.frm.frmap3
-    canvas $MpW.frm.frmap3.cv -width $MAPSCLENGTH -height 15 -relief flat
-    label $MpW.frm.frmap3.cv.val -width 10 -font fixed
-    $MpW.frm.frmap3.cv create line 0 7.5 $MAPSCLENGTH 7.5 -arrow both \
+    # need another frame for the map-main-window layout
+    frame $MpW.frm.frmap3.fr3
+    set fr3 $MpW.frm.frmap3.fr3
+    canvas $fr3.cv -width $MAPSCLENGTH -height 15 -relief flat
+    label $fr3.cv.val -width 10 -font fixed
+    $fr3.cv create line 0 7.5 $MAPSCLENGTH 7.5 -arrow both \
 	    -fill $MAPCOLOUR(mapsel)
-    $MpW.frm.frmap3.cv create line 1 0 1 15 -fill $MAPCOLOUR(mapsel)
+    $fr3.cv create line 1 0 1 15 -fill $MAPCOLOUR(mapsel)
     set l [expr $MAPSCLENGTH-1]
-    $MpW.frm.frmap3.cv create line $l 0 $l 15 -fill $MAPCOLOUR(mapsel)
-    $MpW.frm.frmap3.cv create window [expr $MAPSCLENGTH/2] 7.5 \
-	    -window $MpW.frm.frmap3.cv.val
-    menubutton $MpW.frm.frmap3.mn -text $TXT(change) -menu $MpW.frm.frmap3.mn.m
-    menu $MpW.frm.frmap3.mn.m
+    $fr3.cv create line $l 0 $l 15 -fill $MAPCOLOUR(mapsel)
+    $fr3.cv create window [expr $MAPSCLENGTH/2] 7.5 \
+	    -window $fr3.cv.val
+    menubutton $fr3.mn -text $TXT(change) -menu $fr3.mn.m
+    menu $fr3.mn.m
     set nf 1 ; set nmd "" ; set MAPDISTVALS ""
     set u $SUBDTUNIT ; set f $SUBDSCALE ; set ims [lindex $MAPSCALE 0]
     set imsu [lreplace $MAPSCALE 0 0]
@@ -505,12 +531,14 @@ proc GMInit {} {
 	}
 	set ad [expr $v*$f] ; set t "$v $u"
 	lappend nmd $t ; lappend MAPDISTVALS $ad
-	$MpW.frm.frmap3.mn.m add radiobutton -label $t \
-		-selectcolor $COLOUR(check) -command "MapScaleSet $ad"
+	$fr3.mn.m add radiobutton -label $t -value $t \
+	    -variable MapScaleButton -selectcolor $COLOUR(check) \
+	    -command "MapScaleSet $ad"
 	if { $v == $ims } {
 	    # cannot call MapScaleSet yet!
 	    set nf 0 ; set MapScInitVal $ad
-	    $MpW.frm.frmap3.cv.val configure -text $t
+	    set MapScaleButton $t
+	    $fr3.cv.val configure -text $t
 	}
     }
     if { $nf } {
@@ -520,19 +548,22 @@ proc GMInit {} {
 	} else {
 	    GMMessage "$MESS(badscale): $MAPSCALE"
 	}
-	set MapScInitVal 1
-	$MpW.frm.frmap3.cv.val configure -text "1 $DTUNIT"
+	set MapScInitVal 1 ; set t "1 $DTUNIT"
+	set MapScaleButton $t
+	$fr3.cv.val configure -text "1 $DTUNIT"
     }
     # labels in map scale menu aligned with km values in $MAPDISTVALS    
     set MAPDISTS $nmd
 
-    frame $MpW.frm.frmap3.sep -height 6 -bg $COLOUR(dialbg) -width 5
+    frame $fr3.sep -height 6 -bg $COLOUR(dialbg) -width 5
+    set mb $fr3.sh
+    menubutton $mb -text $TXT(items) -menu $mb.m
+    menu $mb.m
     foreach op "loc disp clr" lb "locate displ clear" exc "GR _ GR" \
 	    fp "SelectApplyMapped SelectApplyUnmapped SelectApplyMapped" \
 	    cm "Locate PutMap UnMap" mode "single many many" {
-	set mn $MpW.frm.frmap3.$op.m
-	menubutton $MpW.frm.frmap3.$op -text $TXT($lb) -menu $mn \
-		-state disabled
+	set mn $mb.m$op
+	$mb.m add cascade -label $TXT($lb) -menu $mn -state disabled
 	menu $mn
 	foreach wh $TYPES {
 	    if { $wh != $exc } {
@@ -540,35 +571,46 @@ proc GMInit {} {
 			-command "$fp $wh $mode $cm"
 	    }
 	}
-    }
-    $MpW.frm.frmap3.loc.m insert 0 command -label $TXT(prevctr) \
-	    -command LocatePrevious
-    set WConf(mapstate) "$MpW.frm.frmap3.loc $MpW.frm.frmap3.clr"
-    set WConf(mapstatetype) "menubutton menubutton"
-    lappend WConf(datastate) [list menubutton $MpW.frm.frmap3.disp]
+    }   
+    $mb.mloc insert 0 command -label $TXT(prevctr) -command LocatePrevious
+    # DJG contribution
+    $mb.m add cascade -label "$TXT(mkgrp)..." -menu $mb.m.mkg
+    menu $mb.m.mkg
+    $mb.m.mkg add command -label $TXT(dispitems) -command "NewGroupFromMap 1" 
+    $mb.m.mkg add command -label $TXT(hiditems) -command "NewGroupFromMap 0"
+    #--
+    set WConf(mapstate) [list [list menu [list [list $mb.m [list 0 2]]]]]
+    lappend WConf(datastate) [list menu [list [list $mb.m 1]]]
 
     frame $MpW.frmi
 
+    set ltypes $TYPES
+    if { $SUPPORTLAPS } { lappend ltypes LAP }
     switch $MWINDOWSCONF {
 	lists {
 	    set WConf(mapdatum) "$MpW.frmi.frcoords.dtitle \
 		    $MpW.frmi.frcoords.ptitle"
-	    lappend WConf(mapstate) $MpW.frmi.frmbs.msave \
-		    $MpW.frmi.frmbs.mprint $MpW.frmi.frmbs.mclear \
-		    $MpW.frmi.frmbs.mftsize
-	    lappend WConf(mapstatetype) button button button menubutton
-	    set WConf(mapstateback) $MpW.frmi.frmbs.mback.m
+	    lappend WConf(mapstate) \
+		[list button \
+		     [list $MpW.frmi.frmbs.msave $MpW.frmi.frmbs.mprint \
+			  $MpW.frmi.frmbs.mclear $MpW.frmi.frmbs.mftsize]]
+	    set WConf(mapstateback) \
+		[list [list menu [list \
+				      [list $MpW.frmi.frmbs.mback.m \
+					   [list 1 2 3]]]]]
 	    set WConf(additemstate) ""
 
 	    set bl [Measure $TXT(backgrnd)]
 	    frame $MpW.frmi.frmbs -borderwidth 2
 	    label $MpW.frmi.frmbs.mtitle -text $TXT(map)
-	    # assuming for each $grid, $grid==$TXT($grid)
 	    menubutton $MpW.frmi.frmbs.grid -textvariable MapPFormat \
 		    -width 8 -menu $MpW.frmi.frmbs.grid.m
-	    menu $MpW.frmi.frmbs.grid.m
-	    bind $MpW.frmi.frmbs.grid <Button-1> \
-		    "FillPFormtMenu $MpW.frmi.frmbs.grid.m ChangeMapPFormat"
+	    menu $MpW.frmi.frmbs.grid.m -postcommand \
+		"FillPFormtMenu $MpW.frmi.frmbs.grid.m ChangeMapPFormat"
+	    set mb $MpW.frmi.frmbs.grdatum
+	    menubutton $mb -textvariable MapPFDatum \
+		    -width [Measure "Astro B4 Sorol Atoll"] -menu $mb.m
+	    menu $mb.m -postcommand "FillDatumMenu $mb.m ChangeMPFDatum"
 
 	    label $MpW.frmi.frmbs.zone -textvariable MapZone -width 4 \
 		    -relief sunken
@@ -653,7 +695,7 @@ proc GMInit {} {
 	    set m $fr0.db.m.import
 	    menu $m
 	    $fr0.db.m add cascade -label "$TXT(import) ..." -menu $m
-	    foreach fmt "GTrackMaker GPX" {
+	    foreach fmt $FILEFORMAT(_type,in,data) {
 		$m add command -label $fmt -command "ImportFile Data $fmt"
 	    }
 	    $fr0.db.m add command -label "$TXT(save) ..." \
@@ -663,7 +705,7 @@ proc GMInit {} {
 	    menu $m
 	    $fr0.db.m add cascade -label "$TXT(export) ..." -menu $m \
 		-state disabled
-	    foreach fmt "GPX" {
+	    foreach fmt $FILEFORMAT(_type,out,data) {
 		$m add command -label $fmt -command "ExportFile all $fmt Data"
 	    }
 	    #--
@@ -671,7 +713,7 @@ proc GMInit {} {
 		    -command "ToggleWindow .lsw $LPOSX $LPOSY"
 	    $fr0.db.m add command -label "$TXT(search) ..." -command Search \
 		    -state disabled
-	    foreach i $TYPES {
+	    foreach i $ltypes {
 		set m $fr0.db.m.e$i
 		$fr0.db.m add cascade -label $TXT(name$i) -menu $m
 		menu $m
@@ -683,60 +725,27 @@ proc GMInit {} {
 		    set im 3
 		    set imp importgrels ; set impst disabled
 		    set imppr ImportGREls
-		    # incorporates VR contribution
-		    set ifmts "GPStrans GD2 GPX Fugawi EasyGPS"
 		    set acts "save saveels"
 		    set prs "SaveFile SaveGREls"
-		    set exp exportels
-		    # includes VR contribution
-		    set exppr ExportGREls ; set efmts "GPStrans GPX Ozi"
-		    if { $GSHPVersion != "" } {
-			lappend efmts Shapefile
-		    }
-		} else {
+		    set exp exportels ; set exppr ExportGREls
+		} elseif { $i != "LAP" } {
 		    set tits [list $TXT(new$i) $TXT(clearall) $TXT(count)]
 		    set coms "NewItem ClearList Count"
 		    set sts "normal disabled normal"
 		    set im 2
 		    set imp import ; set impst normal
 		    set imppr ImportFile
-		    set ifmts "GPStrans GD2 GPX MapSend"
 		    set acts save ; set prs SaveFile
-		    set exp export
-		    # includes VR contribution
-		    set exppr ExportFile
-		    set efmts "GPStrans GPX MapSend"
-		    switch $i {
-			WP {
-			    lappend efmts Ozi
-			    # incorporates VR contribution
-			    lappend ifmts Fugawi EasyGPS
-			}
-			RT {
-			    lappend ifmts MapGuide
-			}
-			TR {
-			    lappend efmts Ozi
-			    lappend ifmts IGC
-			    if { $MYGPS == "Garmin" } {
-				lappend ifmts NMEA
-			    }
-			}
-			LN {
-			    set ifmts "" ; set efmts ""
-			}
-		    }
-		    if { $i != "LN" && $MYGPS == "Magellan" } {
-			# based on MGM contribution
-			lappend ifmts Meridian
-			lappend efmts Meridian
-		    }
-		    if { $GSHPVersion != "" } {
-			lappend ifmts Shapefile
-			lappend efmts Shapefile
-		    }
-		    set ifmts [lsort -dictionary $ifmts]
-		    set efmts [lsort -dictionary $efmts]
+		    set exp export ; set exppr ExportFile
+		} else {
+		    # LAP
+		    set tits [list $TXT(clearall) $TXT(count)]
+		    set coms "ClearList Count"
+		    set sts "disabled normal"
+		    set im 1
+		    set imp "" ; set impst "" ; set imppr ""
+		    set acts save ; set prs SaveFile
+		    set exp "" ; set exppr ""
 		}
 		foreach l $tits c $coms s $sts {
 		    $m add command -label $l -command "$c $i" -state $s
@@ -745,7 +754,7 @@ proc GMInit {} {
 		    $m insert $im cascade -label "$TXT($ip) ..." \
 			    -menu $m.$ip -state $s
 		    menu $m.$ip
-		    foreach fmt $ifmts {
+		    foreach fmt $FILEFORMAT(_type,in,$i) {
 			regsub -all { } $fmt {_} gfmt
 			$m.$ip add command -label $fmt -command "$p $i $gfmt"
 		    }
@@ -764,7 +773,7 @@ proc GMInit {} {
 		    $m insert $im cascade -label "$TXT($ep) ..." \
 			    -menu $m.$ep -state disabled
 		    menu $m.$ep
-		    foreach fmt $efmts {
+		    foreach fmt $FILEFORMAT(_type,out,$i) {
 			regsub -all { } $fmt {_} gfmt
 			$m.$ep add cascade -label $fmt -menu $m.$ep.f$gfmt
 			menu $m.$ep.f$gfmt
@@ -821,8 +830,8 @@ proc GMInit {} {
 		    -command "SetOptions ; \
 		    $fr0.options configure -state normal"
 
-	    frame $MpW.frm.frmap4 -borderwidth 2
-	    set fr4 $MpW.frm.frmap4
+	    frame $MpW.frm.frmap3.fr4 -borderwidth 1 -bg $COLOUR(messbg)
+	    set fr4 $MpW.frm.frmap3.fr4
 
 	    menubutton $fr4.datum -textvariable Datum -menu $fr4.datum.m \
 		    -width [Measure "Astro B4 Sorol Atoll"]
@@ -836,22 +845,27 @@ proc GMInit {} {
 	    label $fr4.transf -textvariable MapTransfTitle -relief raised \
 		    -width 20
 
-	    frame $fr4.sep -height 6 -bg $COLOUR(dialbg) -width 5
-	    menubutton $fr4.grid -textvariable MapPFormat -width 8 \
-		    -menu $fr4.grid.m
-	    menu $fr4.grid.m
-	    bind $fr4.grid <Button-1> \
-		    "FillPFormtMenu $fr4.grid.m ChangeMapPFormat"
-	    label $fr4.zone -textvariable MapZone -width 4 -relief sunken
-	    label $fr4.xcoord -textvariable XCoord -width 11 -relief sunken
-	    label $fr4.ycoord -textvariable YCoord -width 11 -relief sunken
+	    frame $MpW.frm.frmap3.fr5 -borderwidth 1 -bg $COLOUR(messbg)
+	    set fr5 $MpW.frm.frmap3.fr5
+
+	    menubutton $fr5.grid -textvariable MapPFormat -width 8 \
+		    -menu $fr5.grid.m
+	    menu $fr5.grid.m \
+		    -postcommand "FillPFormtMenu $fr5.grid.m ChangeMapPFormat"
+	    label $fr5.zone -textvariable MapZone -width 4 -relief sunken
+	    label $fr5.xcoord -textvariable XCoord -width 11 -relief sunken
+	    label $fr5.ycoord -textvariable YCoord -width 11 -relief sunken
+	    menubutton $fr5.grdat -textvariable MapPFDatum \
+		    -width [Measure "Astro B4 Sorol Atoll"] -menu $fr5.grdat.m
+	    menu $fr5.grdat.m \
+		    -postcommand "FillDatumMenu $fr5.grdat.m ChangeMPFDatum"
 
 	    set WConf(mapdatum) "$fr4.datum $fr4.proj"
-	    set WConf(mapposition) $fr4 ; set WConf(mapposprev) sep
-	    set WConf(mapposside) left
-	    lappend WConf(mapstate) $fr0.map.m
-	    lappend WConf(mapstatetype) menu
-	    set WConf(mapstateback) $fr0.map.mback
+	    lappend WConf(mapstate) \
+		[list menu [list [list $fr0.map.m [list 0 1 3 4]]]]
+	    set WConf(mapstateback) \
+		[list [list menu [list \
+				      [list $fr0.map.mback [list 1 2 3]]]]]
 	    set WConf(additemstate) $fr0.db.m.e
 	    lappend WConf(datastate) \
 		    [list menu [list [list $fr0.db.m [list 2 3 5]]]] \
@@ -883,6 +897,7 @@ proc GMInit {} {
     menu $mn.ell.mn \
 	    -postcommand "FillEllipsoidMenu $mn.ell.mn OpenEllipsoid"
     $mn.ell add cascade -label "$TXT(open)..." -menu $mn.ell.mn
+    $mn add command -label $TXT(syusrmenu) -command SymbolCustomMenu
 
     if { $SmallLogo == "" } {
 	menubutton $gtit -text "GPS Manager" -menu $gtit.m
@@ -895,7 +910,10 @@ proc GMInit {} {
 
     set tw [Measure $TXT(nameWP)]
     set bw [Measure $TXT(clear)]
-    foreach i $TYPES {
+    foreach i $ltypes {
+	set EdWindow($i) .gm$i ; set ListInds($i) ""
+	set GMEd($i,Show) 0
+
 	set n $TXT(name$i)
 	set List($i) $LsW.frl$i.frl.box
 	frame $LsW.frl$i -borderwidth 2
@@ -927,59 +945,28 @@ proc GMInit {} {
 	    set im 4
 	    set imp importgrels ; set impst disabled
 	    set imppr ImportGREls
-	    # incorporates VR contribution
-	    set ifmts "GPStrans GD2 GPX Fugawi EasyGPS"
 	    set acts "save saveels"
 	    set prs "SaveFile SaveGREls"
-	    set exp exportels
-	    # includes VR contribution
-	    set exppr ExportGREls ; set efmts "GPStrans GPX Ozi"
-	    if { $GSHPVersion != "" } {
-		lappend efmts Shapefile
-	    }
-	} else {
+	    set exp exportels ; set exppr ExportGREls
+	} elseif { $i != "LAP" } {
 	    set tits [list $TXT(new$i) $TXT(clearall) "$TXT(load) ..." \
 		               $TXT(count)]
 	    set coms "NewItem ClearList LoadFile Count"
 	    set sts "normal disabled normal normal"
 	    set im 3
 	    set imp import ; set impst normal
-	    set imppr ImportFile ; set ifmts "GPStrans GD2 GPX MapSend"
+	    set imppr ImportFile
 	    set acts save ; set prs SaveFile
-	    set exp export
-	    # includes VR contribution
-	    set exppr ExportFile ; set efmts "GPStrans GPX MapSend"
-	    switch $i {
-		WP {
-		    lappend efmts Ozi
-		    # incorporates VR contribution
-		    lappend ifmts Fugawi EasyGPS
-		}
-		RT {
-		    lappend ifmts MapGuide
-		}
-		TR {
-		    lappend efmts Ozi
-		    lappend ifmts IGC
-		    if { $MYGPS == "Garmin" } {
-			lappend ifmts NMEA
-		    }
-		}
-		LN {
-		    set ifmts "" ; set efmts ""
-		}
-	    }
-	    if { $i != "LN" && $MYGPS == "Magellan" } {
-		# based on MGM contribution
-		lappend ifmts Meridian
-		lappend efmts Meridian
-	    }
-	    if { $GSHPVersion != "" } {
-		lappend ifmts Shapefile
-		lappend efmts Shapefile
-	    }
-	    set ifmts [lsort -dictionary $ifmts]
-	    set efmts [lsort -dictionary $efmts]
+	    set exp export ; set exppr ExportFile
+	} else {
+	    # LAP
+	    set tits [list $TXT(clearall) "$TXT(load) ..." $TXT(count)]
+	    set coms "ClearList LoadFile Count"
+	    set sts "disabled normal normal"
+	    set im 2
+	    set imp "" ; set impst "" ; set imppr ""
+	    set acts save ; set prs SaveFile
+	    set exp "" ; set exppr ""
 	}
 	set m $LsW.frl$i.frb.file.m
 	foreach l $tits c $coms s $sts {
@@ -988,7 +975,7 @@ proc GMInit {} {
 	foreach ip $imp s $impst p $imppr {
 	    $m insert $im cascade -label "$TXT($ip) ..." -menu $m.$ip -state $s
 	    menu $m.$ip
-	    foreach fmt $ifmts {
+	    foreach fmt $FILEFORMAT(_type,in,$i) {
 		$m.$ip add command -label $fmt -command "$p $i $fmt"
 	    }
 	    incr im
@@ -1005,7 +992,7 @@ proc GMInit {} {
 	    $m insert $im cascade -label "$TXT($ep) ..." \
 		    -menu $m.$ep -state disabled
 	    menu $m.$ep
-	    foreach fmt $efmts {
+	    foreach fmt $FILEFORMAT(_type,out,$i) {
 		$m.$ep add cascade -label $fmt -menu $m.$ep.f$fmt
 		menu $m.$ep.f$fmt
 		$m.$ep.f$fmt add command -label $TXT(all) \
@@ -1016,23 +1003,11 @@ proc GMInit {} {
 	    incr im
 	}
     }
-    # cannot use $TYPES here!!!
-    bind $LsW.frlWP.frl.box <Button-3> {
-	ToggleDisplayItem WP [%W nearest %y]
+    foreach t $TYPES {
+	bind $LsW.frl$t.frl.box <Button-3> \
+	    "ToggleDisplayItem $t \[%W nearest %y\]"
     }
-    bind $LsW.frlRT.frl.box <Button-3> {
-	ToggleDisplayItem RT [%W nearest %y]
-    }
-    bind $LsW.frlTR.frl.box <Button-3> {
-	ToggleDisplayItem TR [%W nearest %y]
-    }
-    bind $LsW.frlLN.frl.box <Button-3> {
-	ToggleDisplayItem LN [%W nearest %y]
-    }
-    bind $LsW.frlGR.frl.box <Button-3> {
-	ToggleDisplayItem GR [%W nearest %y]
-    }
-    
+
     frame $RcW.frgps -borderwidth 2
     if { $MYGPS == "Garmin" } {
 	global MyProdDescr
@@ -1056,15 +1031,14 @@ proc GMInit {} {
     menubutton $fprt.title -text $TXT(protcl) -menu $fprt.title.m
     menu $fprt.title.m
     set GPSProtocolExt $TXT($GPSProtocol)
-    foreach p $RECPROTOCOLS {
+    foreach p $RECPROTOCOLS($MYGPS) {
 	$fprt.title.m add command -label $TXT($p) \
 		-command "GPSChangeProtocol $p"
-	$RcMenu.pm add radiobutton -label $TXT($p) \
-		-selectcolor $COLOUR(check) -command "GPSChangeProtocol $p"
+	$RcMenu.pm add radiobutton -label $TXT($p) -variable GPSProtocol \
+	    -value $p -selectcolor $COLOUR(check) \
+	    -command "GPSChangeProtocol $p"
     }
-    $RcMenu.pm invoke 0
     label $fprt.prot -textvariable GPSProtocolExt -width 16
-
     
     if { $MYGPS != "Magellan" } {
 	frame $RcW.frgps.rlt -borderwidth 2 -bg $COLOUR(messbg)
@@ -1075,8 +1049,9 @@ proc GMInit {} {
 	$RcMenu.rtm add command -label $TXT(getlog) \
 		-command GPSRealTimeLogOnOff
 	$RcMenu.rtm add command -label $TXT(dolog) -command { set PVTState on }
-	$RcMenu.rtm add command -label $TXT(animation) \
-		-command { set RealTimeLogAnim 1 }
+	$RcMenu.rtm add checkbutton -label $TXT(animation) \
+	        -offvalue 0 -onvalue 1 -variable RealTimeLogAnim \
+	        -selectcolor $COLOUR(check)
 	# Travel entry added below if Garmin
 
 	button $frlt.onoff -text $TXT(getlog) -width 10 \
@@ -1123,26 +1098,36 @@ proc GMInit {} {
     set r [Measure $TXT(nameWP)]
     frame $RcW.frget -borderwidth 2
     frame $RcW.frget.frget1
+    # the following frame only children are the buttons for types in $RECTYPES
     frame $RcW.frget.frget2
     label $RcW.frget.frget1.title -text $TXT(get)
+    # the following menu only entries are for types in $RECTYPES
     menu $RcMenu.gm
+    set gmlst ""
     $RcMenu add cascade -label $TXT(get) -state disabled -menu $RcMenu.gm
     checkbutton $RcW.frget.frget1.displ -text $TXT(mapitems) \
 	    -variable GetDispl -onvalue 1 -offvalue 0 \
 	    -selectcolor $COLOUR(check)
 
+    # the following frame only children are the buttons for types in $RECTYPES
     frame $RcW.frput -borderwidth 2
     label $RcW.frput.title -text $TXT(put)
+    # the following menu only entries are for types in $RECTYPES
     menu $RcMenu.ptm
+    set ptmlst ""
     $RcMenu add cascade -label $TXT(put) -state disabled -menu $RcMenu.ptm
+    set gmix 0 ; set ptmix 0
     foreach i $RECTYPES {
 	set n $TXT(name$i)
 	button $RcW.frget.frget2.get$i -text $n -width $r -state disabled \
-	       -command "GetFromGPS $i ; \
+	    -command "GetFromGPS $i ; \
 	                 $RcW.frget.frget2.get$i configure -state normal"
-	$RcMenu.gm add command -label $n -command "GetFromGPS $i"
+	$RcMenu.gm add command -label $n -command "GetFromGPS $i" \
+	    -state disabled
+	lappend gmlst rec,getmn,$i $gmix
+	incr gmix
 	# BSB contribution: no IC put button
-	if { $i != "IC" } {
+	if { $i != "IC" && $i != "LAP" } {
 	    menubutton $RcW.frput.put$i -text $n -width $r -state disabled \
 		    -menu $RcW.frput.put$i.m
 	    menu $RcW.frput.put$i.m
@@ -1151,13 +1136,26 @@ proc GMInit {} {
 	    $RcW.frput.put$i.m add command -label "$TXT(select) ..." \
 		    -command "PutOnGPS select $i"
 	    menu $RcMenu.ptm.m$i
-	    $RcMenu.ptm add cascade -label $n -menu $RcMenu.ptm.m$i
+	    $RcMenu.ptm add cascade -label $n -menu $RcMenu.ptm.m$i \
+		    -state disabled
+	    lappend ptmlst rec,putmn,$i $ptmix
+	    incr ptmix
 	    $RcMenu.ptm.m$i add command -label $TXT(all) \
 		    -command "PutOnGPS all $i"
 	    $RcMenu.ptm.m$i add command -label "$TXT(select) ..." \
 		    -command "PutOnGPS select $i"
+	    if { $AutoNumRts && $i == "RT" } {
+		$RcW.frput.put$i.m add command -label $TXT(numberfrom0) \
+		    -command ResetAutoNumberRT
+		$RcMenu.ptm.m$i add command -label $TXT(numberfrom0) \
+		    -command ResetAutoNumberRT
+	    }
 	}
     }
+    array set WConf $gmlst
+    array set WConf $ptmlst
+    set WConf(rec,getbs) [winfo children $RcW.frget.frget2]
+    set WConf(rec,putbs) [winfo children $RcW.frput]
     # BSB contribution
     if { $MYGPS == "Lowrance" } {
 	frame $RcW.frinfo -borderwidth 2
@@ -1197,12 +1195,13 @@ proc GMInit {} {
     pack $MpW.frm.frmap2.mhscr -padx 2 -side left -fill x -expand true
     pack $MpW.frm.frmap2.fill -side left
     pack $MpW.frm.frmap2 -pady 0 -fill x -expand true
-    pack $MpW.frm.frmap3.cv $MpW.frm.frmap3.mn -side left -padx 2
-    pack $MpW.frm.frmap3.sep $MpW.frm.frmap3.loc -side left -padx 10
-    pack $MpW.frm.frmap3.disp $MpW.frm.frmap3.clr -side left -padx 2
-    pack $MpW.frm.frmap3 -pady 2
 
-    foreach i $TYPES {
+    grid config $fr3.cv -row 0 -column 0 -sticky nesw
+    grid config $fr3.mn -row 0 -column 1 -sticky nesw -padx 2
+    grid config $fr3.sep -row 0 -column 2 -sticky nesw -padx 10
+    grid config $fr3.sh -row 0 -column 3 -sticky nesw -padx 5
+
+    foreach i $ltypes {
 	pack $LsW.frl$i.frl.box $LsW.frl$i.frl.bscr -side left \
 		-fill y -expand true
 	pack $LsW.frl$i.frb.file -side top -pady 5
@@ -1213,11 +1212,15 @@ proc GMInit {} {
 
     switch $MWINDOWSCONF {
 	lists {
+	    grid $fr3
+	    pack $MpW.frm.frmap3 -pady 2
+
 	    pack $defsmnbt $MpW.frmi.frcoords.transf \
 		    $MpW.frmi.frcoords.ttitle $MpW.frmi.frcoords.proj \
 		    $MpW.frmi.frcoords.ptitle $MpW.frmi.frcoords.datum \
 		    $MpW.frmi.frcoords.dtitle -pady 2 -side bottom
 	    pack $MpW.frmi.frmbs.mtitle $MpW.frmi.frmbs.grid \
+		    $MpW.frmi.frmbs.grdatum \
 		    $MpW.frmi.frmbs.zone $MpW.frmi.frmbs.xcoord \
 		    $MpW.frmi.frmbs.ycoord $MpW.frmi.frmbs.msave \
 		    $MpW.frmi.frmbs.mprint $MpW.frmi.frmbs.mback \
@@ -1229,11 +1232,24 @@ proc GMInit {} {
 	    pack $MpW.frm $MpW.frmi -side left -fill y
 	}
 	map {
-	    pack $fr4.datum $fr4.proj $fr4.transf $fr4.sep $fr4.grid \
-		    $fr4.zone $fr4.xcoord $fr4.ycoord -side left -padx 3
-	    pack $fr4 -pady 2
+	    grid config $fr4.datum -row 0 -column 0 -sticky nesw
+	    grid config $fr4.proj -row 0 -column 1 -sticky nesw -padx 3
+	    grid config $fr4.transf -row 0 -column 2 -sticky nesw -padx 3
+
+	    grid config $fr5.grid -row 0 -column 0 -sticky nesw
+	    grid config $fr5.zone -row 0 -column 1 -sticky nesw -padx 3
+	    grid config $fr5.xcoord -row 0 -column 2 -sticky nesw -padx 3
+	    grid config $fr5.ycoord -row 0 -column 3 -sticky nesw -padx 3
+	    grid config $fr5.grdat -row 1 -column 0 -columnspan 4 -sticky nesw
+
+	    grid config $fr3 -row 0 -column 0
+	    grid config $fr4 -row 1 -column 0
+	    grid config $fr5 -row 0 -column 1 -rowspan 2 -padx 10
+
+	    pack $MpW.frm.frmap3 -pady 2
 	    pack $MpW.frm
 	    pack $MpW
+
 	    pack .lsw.frbl.frb.load .lsw.frbl.frb.import .lsw.frbl.frb.save \
 		.lsw.frbl.frb.export .lsw.frbl.frb.search -side left -padx 3
 	    pack .lsw.frbl.frb $LsW -side top -pady 3
@@ -1258,7 +1274,7 @@ proc GMInit {} {
     pack $RcW.frput.title -side top
     # BSB contribution: no IC put button
     foreach i $RECTYPES {
-	if { $i != "IC" } { 
+	if { $i != "IC" && $i != "LAP" } { 
 	    pack $RcW.frput.put$i -side left -pady 2
 	}
     }
@@ -1266,9 +1282,9 @@ proc GMInit {} {
     switch $MWINDOWSCONF {
 	lists {
 	    pack .lsw.frmb.dtitle .lsw.frmb.datum -pady 2 -side top
-	    pack .lsw.frmb.frb.sep .lsw.frmb.frb.load  .lsw.frmb.frb.save \
-		    .lsw.frmb.frb.search .lsw.frmb.frb.sep2 -side top -pady 3 \
-		    -fill x
+	    pack .lsw.frmb.frb.sep .lsw.frmb.frb.load .lsw.frmb.frb.import \
+		.lsw.frmb.frb.save .lsw.frmb.frb.export .lsw.frmb.frb.search \
+		.lsw.frmb.frb.sep2 -side top -pady 3 -fill x
 	    pack .lsw.frmb.frb -pady 20 -side top
 	    pack .lsw.frmb.opt -pady 5 -side bottom
 	    pack .lsw.frmb.sep -pady 5 -side bottom -fill x
@@ -1341,13 +1357,15 @@ proc GMStart {} {
 	    TYPES PROCTYPES CursorsChanged MAPWIDTH MAPHEIGHT MAPW2 MAPH2 \
 	    AzimuthDegrees MapEmpty DataDefault DataIndex PositionFormat \
 	    DEFAULTSYMBOL DEFAULTDISPOPT Datum EMPTYSTR AnimNo \
-	    BalloonX BalloonY USERDIR UProjs UPrGrFile RealTimeLogOn \
-	    RealTimeLogAnim PVTState UDtElFile USvState USvData USvMap \
+	    BalloonX BalloonY USERDIR UProjs UFile RealTimeLogOn \
+	    RealTimeLogAnim PVTState USvState USvData USvMap \
 	    BITMAPS SERIALPORT DEFSPORT tcl_platform DEFTRTWIDTH DEFTTRWIDTH \
 	    DEFTLNWIDTH GPSProtocol RECPROTOCOLS GetDispl UNIX MAPCOLOUR \
 	    MapFont MAPFONTSIZE Travelling Travel TRAVELFONTSIZE NAMEWIDTH \
-	    MapMeasure DEFMAPPROJ DEFMAPPFRMT ALTUNIT AltUnit GMEd PlotWNo
-	    
+	    MapMeasure DEFMAPPROJ DEFMAPPFRMT ALTUNIT AltUnit GMEd PlotWNo \
+	    FILEFORMAT DEFTRECPROTOCOL MapPFDatum DEFMAPPFDATUM \
+	    RealTimeGettingFix
+
     # BSB contribution: globals needed for Lowrance receivers
     global UnusedICInx UnusedWPInx MAX MYGPS
 
@@ -1355,31 +1373,22 @@ proc GMStart {} {
     if { $UNIX && $SERIALPORT == "" } {
 	set SERIALPORT $DEFSPORT
     }
-    set GPSProtocol [lindex $RECPROTOCOLS 0]
+
+    if { $MYGPS == "Garmin" } {
+	set GPSProtocol $DEFTRECPROTOCOL
+    } else { set GPSProtocol [lindex $RECPROTOCOLS($MYGPS) 0] }
     set GetDispl 0
 
     # projections, datums and ellipsoids
-    if { [file readable [set UDtElFile [file join $USERDIR datums]]] } {
+    if { [file readable [set UFile(datum,ell) [file join $USERDIR datums]]] } {
 	SetUpUserDatums
     }
     SetUpNZGrids
     set UProjs {}
-    if { [file readable [set UPrGrFile [file join $USERDIR projs]]] } {
+    if { [file readable [set UFile(proj,grid) [file join $USERDIR projs]]] } {
 	SetupUserProjs
     }
 
-    # map
-    set MapEmpty 1 ; set MapMeasure ""
-    set MAPW2 [expr $MAPWIDTH/2.0] ; set MAPH2 [expr $MAPHEIGHT/2.0]
-    set AnimNo 0
-    foreach x "high medium low" y "important normal info" {
-	set MAPCOLOUR(trvwrn$x) $MAPCOLOUR(trvwrn$y)
-    }
-    set MapFont [FixedFont $MAPFONTSIZE]
-
-    MapProjectionIs $DEFMAPPROJ
-    ChangeMapPFormat $DEFMAPPFRMT
-
     # data-base
     set Number(Data) 0
            # the first element in Storage(_) must be identifier storage
@@ -1413,7 +1422,18 @@ proc GMStart {} {
     set DataDefault(LP) [list "" ""]
     set Storage(GR) {GRName GRObs GRConts GRDispl}
     set DataDefault(GR) [list "" "" "" 0]
-    foreach wh $TYPES proc $PROCTYPES {
+    # LAPs should be considered here, just in case support for them is asked
+    #  during the current session
+    set types $TYPES ; set ptypes $PROCTYPES
+    lappend types LAP ; lappend ptypes GMLap
+    # LAPName is formatted date and is not to be edited
+    # LAPStart is the time stamp (list with formatted date and secs)
+    # LAPDispl is here only for uniformity; it should be always 0
+    set Storage(LAP) {LAPName LAPObs LAPStart LAPDur LAPDist LAPBegPosn
+	LAPEndPosn LAPCals LAPTRIx LAPPFrmt LAPDatum LAPDispl}
+    set DataDefault(LAP) [list "" "" "" 0 0 "" "" 0 "" $PositionFormat \
+			      $Datum 0]
+    foreach wh $types proc $ptypes {
 	set Index($wh) 0 ; set Number($wh) 0 ; set Proc($wh) $proc
 	set k 0
 	foreach da $Storage($wh) {
@@ -1430,7 +1450,7 @@ proc GMStart {} {
     set GMEd(LN,ptname) LP
 
     # files
-    set FileTypes {WP RT TR LN GR Data WPDistBear WPNearest RTComp TRComp
+    set FileTypes {WP RT TR LN LAP GR Data WPDistBear WPNearest RTComp TRComp
                    Map Plot MapBkInfo Image PVTData}
     foreach t $FileTypes { set File($t) "" }
     set FDDatum ""
@@ -1439,8 +1459,37 @@ proc GMStart {} {
     set USvData [file join $USERDIR sdata]
     set USvMap [file join $USERDIR smap]
 
+    # foreign formats
+    set FILEFORMAT(_type,in,data) [set FILEFORMAT(_type,out,data) ""]
+    foreach t $TYPES {
+	set FILEFORMAT(_type,in,$t) [set FILEFORMAT(_type,out,$t) ""]
+    }	
+    foreach i [array names FILEFORMAT *,filetype] {
+	regsub {,.*$} $i "" fmt
+	if { $fmt == "GPSMan" || \
+		 ( ! [catch {set dep $FILEFORMAT($fmt,depends)}] && \
+		   ! [eval $dep] ) } {
+	    continue
+	}
+	set ts $FILEFORMAT($fmt,types)
+	if { ! [catch {set FILEFORMAT($fmt,GREls)}] } { lappend ts GR }
+	if { $FILEFORMAT($i) == "data" } {
+	    foreach m $FILEFORMAT($fmt,mode) {
+		lappend FILEFORMAT(_type,$m,data) $fmt
+	    }
+	}
+	foreach m $FILEFORMAT($fmt,mode) {
+	    foreach t $ts {
+		lappend FILEFORMAT(_type,$m,$t) $fmt
+	    }
+	}
+    }
+    foreach i [array names FILEFORMAT _type,*] {
+	set FILEFORMAT($i) [lsort -dictionary $FILEFORMAT($i)]
+    }
+
     # real-time and travel/navigation
-    set RealTimeLogOn 0 ; set RealTimeLogAnim 0
+    set RealTimeLogOn 0 ; set RealTimeLogAnim 0 ; set RealTimeGettingFix 0
     set PVTState abort
     set Travelling 0
     set Travel(font) [FixedFont $TRAVELFONTSIZE]
@@ -1534,6 +1583,20 @@ proc GMStart {} {
 	warn:xtk,cont {[format $TXT(trvwxtk) $Travel(warn,xtk,max)]}
     }
 
+    # map
+    set MapEmpty 1 ; set MapMeasure ""
+    set MAPW2 [expr $MAPWIDTH/2.0] ; set MAPH2 [expr $MAPHEIGHT/2.0]
+    set AnimNo 0
+    foreach x "high medium low" y "important normal info" {
+	set MAPCOLOUR(trvwrn$x) $MAPCOLOUR(trvwrn$y)
+    }
+    set MapFont [FixedFont $MAPFONTSIZE]
+
+    MapProjectionIs $DEFMAPPROJ
+    # order is important here
+    set MapPFDatum $DEFMAPPFDATUM
+    ChangeMapPFormat $DEFMAPPFRMT
+
     # varia
     set AltUnit $ALTUNIT
     set PlotWNo 0
