@@ -17,7 +17,7 @@
 #      along with this program.
 #
 #  File: trtort.tcl
-#  Last change:  17 January 2004
+#  Last change:  17 September 2004
 #
 # Includes contributions by Brian Baulch (baulchb@onthenet.com.au)
 #  marked "BSB contribution"
@@ -153,22 +153,17 @@ proc TRCvCommitRT {w} {
 	    incr n ; set id [format "ZT%04d" $n]
 	}
 	incr n
+	foreach "p pfmt pfdatum" \
+	    [FormatPosition $latd $longd $datum $PositionFormat "" DDD] {
+		break
+	}
 	set data [FormData WP "Name PFrmt Posn Datum Date Alt" \
-		     [list $id $PositionFormat \
-		       [CreatePos [lindex $p 0] [lindex $p 1] \
-		                  $PositionFormat $datum] \
-		       $datum $date [lindex $p 6]]]
+		     [list $id $pfmt $p $pfdatum $date [lindex $p 6]]]
 	set ix [CreateItem WP $data]
 	lappend wps $id
     }
     SlowOpFinish ""
-    set grname [format "TR-RT %d" $n]
-    while { [IndexNamed GR $grname] != -1 } {
-	incr n ; set grname [format "TR-RT %d" $n]
-    }
-    set data [FormData GR "Name Obs Conts" \
-	         [list $grname $TXT(obsTRToRT) [list [list WP $wps]]]]
-    CreateItem GR $data
+    CreateGRFor =TR-RT $TXT(obsTRToRT) [list [list WP $wps]]
     TRCvCancel $w
     if { [winfo exists $EdWindow(RT)] } {
 	set name [NewName RT]
@@ -467,15 +462,18 @@ proc GMTRtoWP {window} {
     if { $noalt } {
 	set sal ""
     } else { set sal [format %.1f [expr 1.0*$sal/$n]] }
-    set p [CreatePos [expr 1.0*$sla/$n] [expr 1.0*$slo/$n] $PositionFormat \
-	       $datum]
+    foreach "p pfmt pfdatum" \
+	[FormatPosition [expr 1.0*$sla/$n] [expr 1.0*$slo/$n] $datum \
+	     $PositionFormat "" DDD] {
+	break
+    }
     set opts {create revert cancel}
     if { $CREATIONDATE } {
 	GMWPoint -1 $opts [FormData WP "PFrmt Posn Datum Alt Date" \
-		[list $PositionFormat $p $datum $sal [Now]]]
+		[list $pfmt $p $pfdatum $sal [Now]]]
     } else {
 	GMWPoint -1 $opts [FormData WP "PFrmt Posn Datum Alt Commt" \
-		[list $PositionFormat $p $datum $sal [DateCommt [Now]]]]
+		[list $pfmt $p $pfdatum $sal [DateCommt [Now]]]]
     }
     return
 }
@@ -502,8 +500,7 @@ proc GMGRtoWP {window} {
 	if { $datum == "" } {
 	    set datum $WPDatum($ix)
 	} elseif { $WPDatum($ix) != $datum } {
-	    set p [ConvertDatum [lindex $p 0] [lindex $p 1] \
-		    $WPDatum($ix) $datum DDD]
+	    set p [ToDatum [lindex $p 0] [lindex $p 1] $WPDatum($ix) $datum]
 	}
 	set sla [expr $sla+[lindex $p 0]] ; set slo [expr $slo+[lindex $p 1]]
 	if { [regexp {[0-9\.]+} [set a $WPAlt($ix)]] } {
@@ -515,15 +512,18 @@ proc GMGRtoWP {window} {
     if { $noalt } {
 	set sal ""
     } else { set sal [format %.1f [expr 1.0*$sal/$n]] }
-    set p [CreatePos [expr 1.0*$sla/$n] [expr 1.0*$slo/$n] $PositionFormat \
-	       $datum]
+    foreach "p pfmt pfdatum" \
+	[FormatPosition [expr 1.0*$sla/$n] [expr 1.0*$slo/$n] $datum \
+	     $PositionFormat "" DDD] {
+	break
+    }
     set opts {create revert cancel}
     if { $CREATIONDATE } {
 	GMWPoint -1 $opts [FormData WP "PFrmt Posn Datum Alt Date" \
-		[list $PositionFormat $p $datum $sal [Now]]]
+		[list $pfmt $p $pfdatum $sal [Now]]]
     } else {
 	GMWPoint -1 $opts [FormData WP "PFrmt Posn Datum Alt Commt" \
-		[list $PositionFormat $p $datum $sal [DateCommt [Now]]]]
+		[list $pfmt $p $pfdatum $sal [DateCommt [Now]]]]
     }
     return
 }
